#include "LK.h"
#include "Hashing.h"
#include "Heap.h"

/*      
 * The AllocateStructures function allocates all necessary 
 * structures except nodes and candidates.
 */

#define Free(s) { free(s); s = 0; }

void AllocateStructures()
{
    long i;

    Free(Heap);
    Free(BestTour);
    Free(BetterTour);
    Free(HTable);
    Free(Rand);
    Free(CacheSig);
    Free(CacheVal);
    Free(SwapStack);

    MakeHeap(Dimension);
    assert(BestTour = (long *) calloc((1 + Dimension), sizeof(long)));
    assert(BetterTour = (long *) calloc((1 + Dimension), sizeof(long)));
    assert(HTable = (HashTable *) malloc(sizeof(HashTable)));
    HashInitialize(HTable);
    srand(Seed);
    assert(Rand = (unsigned int *) malloc((Dimension + 1) * sizeof(long)));
    for (i = 1; i <= Dimension; i++)
	Rand[i] = rand();
    srand(Seed);
    if (WeightType != EXPLICIT) {
	for (i = 1; i <= Dimension; i *= 2);
	assert(CacheSig = (long *) calloc(i, sizeof(long)));
	assert(CacheVal = (long *) calloc(i, sizeof(long)));
    }
    AllocateSegments();
    assert(SwapStack =
	   (SwapRecord *) malloc((Dimension + 10) * sizeof(SwapRecord)));
}

/*      
 * The AllocateSegments function allocates the segments of the two-level 
 * tree.
 */

void AllocateSegments()
{
    Segment *S, *SPrev;
    long i;

    FreeSegments();
    GroupSize = sqrt(Dimension);
    Groups = 0;
    for (i = Dimension, SPrev = 0; i > 0; i -= GroupSize, SPrev = S) {
	assert(S = (Segment *) malloc(sizeof(Segment)));
	S->Rank = ++Groups;
	if (!SPrev)
	    FirstSegment = S;
	else
	    Link(SPrev, S);
    }
    Link(S, FirstSegment);
}
