//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
// Copyright 1997 INRA (Institut National de la Recherche Agronomique).
// This file is distributed under the terms of the Q Public License version 1.0.
// 
// $Id: BJM_OR.h,v 1.23 2003/12/19 15:12:15 degivry Exp $
//
// Description : Prototype de la classe BJM_OR.
// Divers : 
//-----------------------------------------------------------------------------

#ifndef _BJM_OR_H
#define _BJM_OR_H

#include "BioJeuMerged.h"


/** Classe de la fusion sur l'ordre. */  
class BJM_OR : public BioJeuMerged {
 public:
  
  /** Constructeur. */
  BJM_OR();
  
  
  /** Constructeur
      @param cartag accs  l'ensemble des informations du systme. 
      @param id le numro du jeu.
      @param cross type de jeu de donne(Mor).
      @param nm nombre de marqueurs.
      @param bitjeu champ de bit du jeu de donnes.
      @param gauche jeu de donnes.
      @param droite jeu de donnes. 
  */
  BJM_OR(CartaGenePtr cartag,
	 int id,
	 CrossType cross,
	 int nm, 
	 int bitjeu,
	 BioJeu *gauche, 
	 BioJeu *droite);
  
  /** Destructeur. */
  virtual ~BJM_OR();

  /** Retourne un champ des jeux fusionns sur l'ordre(!=CON).
      @return un champ de bits
  */
  inline int BJM_OR::GetBJMO() const
    {
      return (BJgauche->GetBJMO() | BJdroite->GetBJMO());
    };
  
  /** Calcul du maximum de vraisemblance par EM.
      A partir de la carte reue, sont composes les deux cartes propres aux deux jeux de donnes fusionns. Le maximum de vraisemblance respectif est calcul. Le rsultat est la somme des deux valeurs intemdiaires.
      @param data la carte
      @return le maximum de vraissemblance
  */
  double ComputeEM(Carte *data);


  /** Calcul de la vraisemblance maximum par EM.
      Une interface plus sophistique du calcul de la vraisemblance par EM:
      On converge jusqu' epsilon1. Si la vraisemblance est infrieure 
      threshold on laisse tomber, sinon on converge  epsilon2.
      @param data la carte
      @param threshold niveau a partir duquel on convergera a epsilon2
      @return le maximum de vraissemblance
  */
  double ComputeEMS (Carte *data, 
		     double threshold);
  
  /** Calcul, affichage et stockage des groupes.
      On tablit un graphe dont les noeuds sont les marqueurs. Il existe une arrte entre deux marqueurs, si le critere de lod > seuil est vrifi. Puis on extrait les composantes connexes qui sont quivalentes aux groupes.
      @param disthres PAS UTILISE.
      @param lodthres le seuil de LOD 2pt.
      @return le nombre de groupes.
    */
  int Groupe(double disthres, double lodthres) const;

  /** Affichage simple d'une carte
      @param data la carte
  */
  void PrintMap(Carte *data) const;

  /** retourne une liste contenant une carte
      @param unit l'unit de distance: 
      'k' = kosambi ou centiray, 
      'h' = haldane ou centiray.
      @param data la carte
      @return une liste
  */
  char *** GetMap(char unit[2], Carte *data) const;

  /** Affichage dtaill d'une carte
      @param data la carte
      @param envers  l'envers ou  l'endroit
      @param dataref la carte de rfrence
  */
  void PrintDMap(Carte *data, int envers, Carte *dataref);

  /** Matrice 2p des distances 5bidon)
      @param unit l'unit de distance: 
      'k' = kosambi ou centiray, 
      'h' = haldane ou centiray.
  */
  void PrintTwoPointsDist(char unit[2]) const {
    BJgauche->PrintTwoPointsDist(unit);
    BJdroite->PrintTwoPointsDist(unit);
  };
    
  void InitContribLogLike2pt() {BJgauche->InitContribLogLike2pt(); BJdroite->InitContribLogLike2pt();}
  double ContribLogLike2pt(int m1) {return BJgauche->ContribLogLike2pt(m1) + BJdroite->ContribLogLike2pt(m1);}
  double ContribLogLike2pt(int m1, int m2) {return BJgauche->ContribLogLike2pt(m1,m2) + BJdroite->ContribLogLike2pt(m1,m2);} 
  double NormContribLogLike2pt(int m1) {return BJgauche->NormContribLogLike2pt(m1) + BJdroite->NormContribLogLike2pt(m1);}
  double NormContribLogLike2pt(int m1, int m2) {return BJgauche->NormContribLogLike2pt(m1,m2) + BJdroite->NormContribLogLike2pt(m1,m2);} 
  double ContribOCB(int m1, int m2) {return BJgauche->ContribOCB(m1,m2) + BJdroite->ContribOCB(m1,m2);} 
  double NormContribOCB(int m1, int m2) {return BJgauche->NormContribOCB(m1,m2) + BJdroite->NormContribOCB(m1,m2);} 

 private:

  /** Calcul des valeurs de points et remplissage des matrices.
      En l'occurence, il s'agit uniquement des LOD 2pts des couples de marqueurs prsents dans les deux jeux fusionns.
  */
  void ComputeTwoPoints(void);

  /** Critre de linkage entre 2 marqueurs.
      @param disthres PAS UTILISE.
      @param lodthres le seuil de LOD 2pt.
      @param k le numro du premier marqueur
      @param l le numro du second marqueur
      @return critre vrifi ou pas.
  */
  inline int GroupTest(double disthres, double lodthres, int k, int l) const 
    {
      return (GetTwoPointsLOD(k, l) >= lodthres);
    }
  ;
  
  /** affichage du seuil de LOD
      @param disthres PAS UTILISE.
      @param lodthres le seuil de LOD 2pt.
  */
  inline void GroupMess(double disthres, double lodthres) const
    {
      sprintf(bouf,"\nLinkage Groups :\n");
      pout(bouf);
      sprintf(bouf,"---------------:\n");
      pout(bouf);
      sprintf(bouf,"LOD threshold=%.2f\n", lodthres);
      pout(bouf);
    }
  ;
    
  /** passage  priv = choix d'implmentation. 
      REMARQUE : Cela signifie que cette caractristique  n'est pas logique dans cette classe,
      mais que pour des raisons d'implmentation, elle figure nanmoins dans cette classe. Elle n'est donc pas documente. 
  */
  int NbMeiose;

  /** passage  priv = choix d'implmentation. */
  doublePtr *TwoPointsFR;

  /** passage  priv = choix d'implmentation. */
  doublePtr *TwoPointsDH;

  /** passage  priv d'une virtuelle hrite = choix d'implmentation. */
  double GetTwoPointsDH(int m1, int m2) const;
  //const {
  //  sprintf(bouf,"Bug, mthode interdite = GetTwoPointsDH!!!\n");
  //  pout(bouf);
  //  return 0.0; 
  //};
  
  /** passage  priv d'une virtuelle hrite = choix d'implmentation. */
  double GetTwoPointsFR(int m1, int m2) const {
    double fr;
    if ((fr = BJgauche->GetTwoPointsFR(m1, m2) +
	 BJdroite->GetTwoPointsFR(m1, m2)) != 0.0) return fr;
    else return Em_Max_Theta;
  };

  /** passage  priv d'une virtuelle hrite = choix d'implmentation. */
  double ComputeExpected(const Carte *data, double *expected) {
    sprintf(bouf,"Bug, mthode interdite = ComputeExpected!\n");
    pout(bouf);
    return 0.0;};

  /** passage  priv d'une virtuelle hrite = choix d'implmentation. */
  void PreparEM(const Carte *data) {
    sprintf(bouf,"Bug, mthode interdite = PreparEM!\n");
    pout(bouf);
  };

  /** passage  priv d'une virtuelle hrite = choix d'implmentation. */
  void NetEM(const Carte *data) {
    sprintf(bouf,"Bug, mthode interdite = NetEM!\n");
    pout(bouf);
  };

  /** passage  priv  = choix d'implmentation. */  
  int Maximize(Carte *data, double *expected) const {
    sprintf(bouf,"Bug, mthode interdite = Maximize!\n");
    pout(bouf);
    return -1;
  };
	
   protected:


  /** Calcul stat. suffisantes pour le 2pt
      @param m1 premier marqueur
      @param m2 second marqueur
      @param ss statistiques suffisantes (n[1..3])
  */
  void Prepare2pt(int m1, int m2,  int *ss) const {
    sprintf(bouf,"Bug, mthode interdite = Prepare2pt!\n");
    pout(bouf);
    return;
  };

  /** Calcul des estimateurs 2pt de vrais. max
      @param par vecteur des parametres du modele (theta,r mis a jour)
      @param ss statistiques suffisantes (n[1..3])
  */
  void Estimate2pt(double *par, int *ss) const  {
    sprintf(bouf,"Bug, mthode interdite = Estimate2pt!\n");
    pout(bouf);
    return;
  };

  /** Calcul de la vraisemblance 2pt etant donnes les parametres du
      modele.
      @param par vecteur des parametres du modele (theta,r)
      @param ss statistiques suffisantes (n[1..3])
  */
  double LogLike2pt(double *par, int *ss) const  {
    sprintf(bouf,"Bug, mthode interdite = LogLike2pt!\n");
    pout(bouf);
    return 0.0;
  };


  /** Calcul du LOD score et de fraction de recombinants 2pt.
      @param m1 le numro du premier marqueur
      @param m2 le numro du second marqueur
      @param epsilon le seuil de convergence
      @param fr la fraction de recombinants(i,o)
      @return le LOD
  */
  double ComputeOneTwoPoints(int m1, 
                             int m2, 
                             double epsilon,
                             double *fr) const;

  /** Accs  une valeur LOD score 2pt.
      @param m1 le numro du premier marqueur
      @param m2 le numro du second marqueur
      @return le lod
  */
  double GetTwoPointsLOD(int m1, int m2) const;
  
		     
};

#endif /* _BJM_OR_H */
