//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
// Copyright 1997 INRA (Institut National de la Recherche Agronomique).
// This file is distributed under the terms of the Q Public License version 1.0.
// 
// $Id: BJS_IC.cc,v 1.16 2003/01/06 13:50:17 bouchez Exp $
//
// Description : BJS_IC.
// Divers :
//-----------------------------------------------------------------------------

#include "BJS_IC.h"
#include "Genetic.h"
#include "System.h"
#include <stdio.h>
#include <string.h>
#include <math.h>

/** liste des genotypes possibles pour chaque phenotype */

const int Possibles[16][4] = {
  {0, 0, 0, 0},  // 0 (aka 0 = 0000 = 0 )
  {0, 0, 0, 0},  // A (aka 1 = 0001 = 1 )
  {1, 0, 0, 0},  // 2 (aka 2 = 0010 = 2 )
  {0, 1, 0, 0},  // 3 (aka 3 = 0011 = 3 )
  {2, 0, 0, 0},  // 4 (aka 4 = 0100 = 4 )
  {0, 2, 0, 0},  // 5 (aka 5 = 0101 = 5 )
  {1, 2, 0, 0},  // H (aka 6 = 0110 = 6 )
  {0, 1, 2, 0},  // D (aka 7 = 0111 = 7 )
  {3, 0, 0, 0},  // B (aka 8 = 1000 = 8 )
  {0, 3, 0, 0},  // 9 (aka 9 = 1001 = 9 )
  {1, 3, 0, 0},  // a (aka a = 1010 = 10)
  {0, 1, 3, 0},  // b (aka b = 1011 = 11)
  {2, 3, 0, 0},  // c (aka c = 1100 = 12)
  {0, 2, 3, 0},  // d (aka d = 1101 = 13) 
  {1, 2, 3, 0},  // C (aka e = 1110 = 14)
  {0, 1, 2, 3}}; // - (aka f = 1111 = 15) 
    
/** nombre de possibilites pour chaque phenotype */
const int NPossibles[16] = {0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4};

/** Nombre de crossing over selon la paire de genotypes */
const int NCross[4][4] = {{0, 1, 1, 2},
			  {1, 0, 2 ,1},
			  {1, 2, 0, 1},
			  {2, 1, 1, 0}};

//-----------------------------------------------------------------------------
// Constructeur.
//-----------------------------------------------------------------------------

BJS_IC::BJS_IC() : BioJeuSingle()
{
  for (int i = 0; i < 4; i++) {
    SourceTo[i] = NULL;
    ToSink[i] = NULL;
  }
}

//-----------------------------------------------------------------------------
// Constructeur.
//-----------------------------------------------------------------------------
// Paramtres :
// - accs  l'ensemble des informations du systme. 
// - le numro du jeu
// - le type
// - le nombre de marqueurs
// - la taille de l'echantillon
// - le champ de bit
// - la matrice de l'chantillon
// Valeur de retour :
//-----------------------------------------------------------------------------

BJS_IC::BJS_IC(CartaGenePtr cartag,
	       int id,
	       charPtr nomjeu,
	       CrossType cross,
	       int nm, 
	       int taille,
	       int bitjeu,
	       int *indmarq,
	       Obs **echantil)
  :BioJeuSingle(cartag,
		id,
                nomjeu,
		cross, 
		nm, 
		taille, 
		bitjeu, 
		indmarq, 
		echantil)
{

  for (int i = 0; i < 4; i++) {
    SourceTo[i] = NULL;
    ToSink[i] = NULL;
  }

  NbMeiose = 2*TailleEchant;
  
  ComputeTwoPoints();
}

//-----------------------------------------------------------------------------
// Destructeur
//-----------------------------------------------------------------------------

BJS_IC::~BJS_IC()
{
}

//-----------------------------------------------------------------------------
// Compatibilite des observations entres individus
//-----------------------------------------------------------------------------
int BJS_IC::Compatible(int numarq1, int numarq2) const
{
  int i;
  enum Obs Obs1, Obs2;
  
  for (i = 0; i< TailleEchant; i++) {
    Obs1 = GetEch(numarq1,i);
    Obs2 = GetEch(numarq2,i);
    if (!(Obs1 & Obs2))  return 0;
  }
  return 1;
}
//-----------------------------------------------------------------------------
// Fusion des observations entres individus
// Cela suppose que la compatibilite a ete verifie AVANT !!!
//-----------------------------------------------------------------------------
void BJS_IC::Merge(int numarq1, int numarq2) const
{
  int i;

  // les deux marqueurs sont ils definis dans le jeux de donnees ?
  if ((Cartage->BitJeuMarq[numarq1] & BitJeu) && (Cartage->BitJeuMarq[numarq2] & BitJeu))
    {
      // Oui, on va fusionner dans le premier
      for (i = 0; i< TailleEchant; i++) {
        Echantillon[IndMarq[numarq1]][i] = 
	    Obs(Echantillon[IndMarq[numarq2]][i] & Echantillon[IndMarq[numarq2]][i]);
      }
    }
}


//-----------------------------------------------------------------------------
//  Calcul de la probabilite de passage par une arete suivant la paire de 
// genotypes relies
//-----------------------------------------------------------------------------
// Paramtres : 
// - les deux genotypes, la proba de rec
// - 
// Valeur de retour : la proba
//-----------------------------------------------------------------------------

inline double BJS_IC::ProbArete(int a, int b, double p) const
{

 int n;
 int c = NCross[a][b];
 double prob = 1.0;

 for (n = 0; n < 2; n++)
   {
     prob *= ((c-- > 0) ? p : (1.0-p));
   }
 return prob;
}


//-----------------------------------------------------------------------------
// Retourne l'esprance du nombre de recombinants.
//-----------------------------------------------------------------------------
// Paramtres :
// - indice du premier marqueur dans le jeu
// - indice du second marqueur dans le jeu
// - le theta
// - le nombre de donnes informatives
// - le loglike (o)
// Valeur de retour : l'esprance du nombre de recombinaisons.
//-----------------------------------------------------------------------------
 
double BJS_IC::EspRec(int m1, 
		      int m2, 
		      double theta, 
		      double *loglike) const
{
  int NbEdges[16][16];
  int i,j,sl,sr;
  double ENRec,LogLike, prob;
  double ProbCross[3];

  
  for (i = 0; i < 16; i++)
    for (j = 0; j < 16; j++) NbEdges[i][j] = 0;
  
  for (i = 1; i <= TailleEchant; i++)
    NbEdges[GetEch(m1, i)][GetEch(m2, i)]++;
        
  LogLike = 0.0;
  ENRec = 0.0;
  
  for (i = 1; i < 15; i++) //observation a gauche, pas les inconnus
    for (j = 1; j < 15; j++) //observation a droite, pas les inconnus
      
      // Si il existe des occurrences de ce type de paires d'observ.
      if (NbEdges[i][j])
	{
	  ProbCross[2] = ProbCross[1] = ProbCross[0] = 0.0;
	  // on calcule les proba de 0,1,2 recombinaisons
	  for (sl = 0; sl< NPossibles[i]; sl++)
	    for (sr= 0; sr < NPossibles[j]; sr++)
	      {
		ProbCross[NCross[Possibles[i][sl]][Possibles[j][sr]]] 
		  += ProbArete(Possibles[i][sl],
			       Possibles[j][sr], theta);
	      }
	  
	  // on en deduit l'esperance du nombre de rec. et donc l'estime
	  // de la fraction de rec. et la vraisemblance 2 pt
	  
	  prob = ProbCross[2]+ ProbCross[1]+ ProbCross[0];
	  ProbCross[2] /= prob;
	  ProbCross[1] /= prob;
	  ProbCross[0] /= prob;

	  ENRec += NbEdges[i][j]*(ProbCross[2]*2 +  ProbCross[1]);
	  LogLike += log10(prob)*NbEdges[i][j];
	}

  *loglike = LogLike;

  return ENRec;
}
//-----------------------------------------------------------------------------
// Calcule le nombre de donnes et 
// le log sous l'hypothese d'independance
//-----------------------------------------------------------------------------
// Paramtres : 
// - les deux numeros de marqueurs, on doit avoir m1< m2
// - le nombre de donnes
// Valeur de retour : le log
//-----------------------------------------------------------------------------

double BJS_IC::LogInd(int m1, 
		      int m2,
		      int *nbdata) const
{
  int NbEdges[16][16];
  int data = 0; // pairs of non unknown
  int i,j,sl,sr;
  double LogLike;
  double ProbCross[3];

  for (i = 0; i < 16; i++)
    for (j = 0; j < 16; j++) NbEdges[i][j] = 0;

  for (i = 1; i <= TailleEchant; i++)
    {
      NbEdges[GetEch(m1, i)][GetEch(m2, i)]++;
      if ((GetEch(m1, i) != Obs1111) && (GetEch(m2, i) != Obs1111))
	data++;
    }

  // On repete le meme calcul en fixant theta a 0.5 (hypothese non lies)
  // bien sur, inutile d'iterer
  LogLike = 0.0;
  
  for (i = 1; i < 15; i++) //observation a gauche
    for (j = 1; j < 15; j++) //observation a droite
      
      // Si il existe des occurrences de ce type de paires d'observ.
      if (NbEdges[i][j])
	{
	  ProbCross[2] = ProbCross[1] = ProbCross[0] = 0.0;
	  // on calcule les proba de 0,1,2 recombinaisons
	  for (sl = 0; sl < NPossibles[i]; sl++)
	    for (sr = 0; sr < NPossibles[j]; sr++)
	      {
		ProbCross[NCross[Possibles[i][sl]][Possibles[j][sr]]] 
		  += ProbArete(Possibles[i][sl],
			       Possibles[j][sr], 0.5);
	      }
	  
	  // on en deduit la vraisemblance 2 pt
	  LogLike += log10(ProbCross[2]+ ProbCross[1]+ ProbCross[0]) *
	    NbEdges[i][j];
	}
  
  *nbdata = 2 * data;
  return LogLike;
}

//-----------------------------------------------------------------------------
// Calcule fraction de rec. et
// LOD scores pour deux marqueurs
//-----------------------------------------------------------------------------
// Paramtres : 
// - les deux numeros de marqueurs, on doit avoir m1< m2
// - epsilon
// - la fraction de reconbinant(i,o)
// Valeur de retour : le lod
//-----------------------------------------------------------------------------

double BJS_IC::ComputeOneTwoPoints(int m1, 
				   int m2, 
				   double epsilon,
				   double *fr) const
{
  int data = 0; // pairs of non unknown
  double ENRec,LogLike,LogLikeInd, PrevLogLike,theta;

  LogLikeInd = LogInd(m1, m2, &data);

  if (!data)  {
    *fr = EM_MAX_THETA;
    return 0.0;
  }
  
  theta = 0.05;
  LogLike = 0.0;

  do {
    PrevLogLike = LogLike;

    ENRec = EspRec(m1, m2, theta, &LogLike);

    theta = ENRec/(data); 
    if (theta > EM_MAX_THETA) theta= EM_MAX_THETA;
  }   while (fabs(LogLike-PrevLogLike) > epsilon);
  

  *fr = theta;
  return(LogLike - LogLikeInd);
}

//-----------------------------------------------------------------------------
//  Etape d'  Expectation  de EM.
//-----------------------------------------------------------------------------
// Paramtres : la carte
// - la carte
// - le vecteur d'Expectation(i,o)
// Valeur de retour : le loglike 
//-----------------------------------------------------------------------------

double BJS_IC::ComputeExpected(const Carte* map, double *expected)
{

 double loglike = 0.0;

 for (int i = 1 ; i <= TailleEchant; i++)  {
   loglike += ComputeSourceTo(i, map);
   ComputeToSink(i, map);
   ComputeOneExpected(i, map, expected);
 }
 
 loglike -= log10(2.0) * TailleEchant * 2.0;
 return (loglike);
}


//-----------------------------------------------------------------------------
//  Calcul des probabilites d'existence d'un chemin
// passant par un sommet depuis l'extremite gauche (source)
//-----------------------------------------------------------------------------
// Paramtres :
// - l'individu 
// - la carte 
// Valeur de retour : le loglike 
//-----------------------------------------------------------------------------

double BJS_IC::ComputeSourceTo(int Ind, const Carte *map) const
{
  int i,j,k;
  int *ordre = map->ordre;
  double norm = 0.0;
  double loglike = 0.0;
  int PhenoL, PhenoR;

  PhenoR = GetEch(ordre[0], Ind);

  for (i = 0; i< NPossibles[PhenoR]; i++)
    SourceTo[Possibles[PhenoR][i]][0] = 1.0;

  for (i = 1; i< map->NbMarqueur; i++)
    {
      SourceTo[0][i] = 0.0;
      SourceTo[1][i] = 0.0;
      SourceTo[2][i] = 0.0;
      SourceTo[3][i] = 0.0;

      PhenoL = PhenoR;
      PhenoR = GetEch(ordre[i], Ind);

      for (j = 0; j< NPossibles[PhenoL]; j++)
	{
	  for (k = 0; k < NPossibles[PhenoR]; k++)
	    SourceTo[Possibles[PhenoR][k]][i] += 
	      (SourceTo[Possibles[PhenoL][j]][i-1]) *
	      ProbArete(Possibles[PhenoL][j],Possibles[PhenoR][k],map->tr[i-1]);
	}
      norm = SourceTo[0][i]+SourceTo[1][i]+SourceTo[2][i]+SourceTo[3][i];
      loglike += log10(norm);
      
      SourceTo[0][i] /= norm;
      SourceTo[1][i] /= norm;
      SourceTo[2][i] /= norm;
      SourceTo[3][i] /= norm;
    }

  return loglike;
}

//-----------------------------------------------------------------------------
// Calcul des probabilites d'existence d'un chemin
// passant par un sommet depuis l'extremite droite (puit)
//-----------------------------------------------------------------------------
// Paramtres : 
// - l'individu
// - la carte 
// Valeur de retour : 
//-----------------------------------------------------------------------------

double BJS_IC::ComputeToSink(int Ind, const Carte *map) const
{
  int i,j,k;
  int *ordre = map->ordre;
  double norm = 0.0;
  double loglike = 0.0;
  int PhenoL,PhenoR;

  PhenoL = GetEch(ordre[map->NbMarqueur - 1], Ind);

  for (i = 0; i< NPossibles[PhenoL]; i++)
    ToSink[Possibles[PhenoL][i]][map->NbMarqueur - 1] = 1.0;

  for (i = (map->NbMarqueur - 2); i >=0; i--)
    {
      ToSink[0][i] = 0.0;
      ToSink[1][i] = 0.0;
      ToSink[2][i] = 0.0;
      ToSink[3][i] = 0.0;

      PhenoR = PhenoL;
      PhenoL = GetEch(ordre[i], Ind);

      for (j = 0; j< NPossibles[PhenoR]; j++)
	{
	  for (k = 0; k < NPossibles[PhenoL]; k++)
	    ToSink[Possibles[PhenoL][k]][i] += 
	      ToSink[Possibles[PhenoR][j]][i+1] * 
	      ProbArete(Possibles[PhenoL][k],Possibles[PhenoR][j],map->tr[i]);
	}
      norm = ToSink[0][i]+ToSink[1][i]+ToSink[2][i]+ToSink[3][i];
      loglike += log10(norm);
      
      ToSink[0][i] /= norm;
      ToSink[1][i] /= norm;
      ToSink[2][i] /= norm;
      ToSink[3][i] /= norm;
    }

  return loglike;
}

//-----------------------------------------------------------------------------
//  Etape elementaire d'  Expectation  de EM.
//-----------------------------------------------------------------------------
// Paramtres : 
// - l'individu 
// - la carte
// - le vecteur d'expectation
// - 
// Valeur de retour : 
//-----------------------------------------------------------------------------

void BJS_IC::ComputeOneExpected(int Ind, 
				const Carte* map, 
				double *expected) const
{
 int j,sl,sr;
 int *ordre = map->ordre;
 int PhenoR = GetEch(ordre[0], Ind);
 int PhenoL;
 double prob;
 double ProbCross[3];

 for (j = 0; j< map->NbMarqueur - 1; j++)
   {
     ProbCross[2] = ProbCross[1] = ProbCross[0] = 0.0;
     PhenoL = PhenoR;
     PhenoR = GetEch(ordre[j+1], Ind);

     for (sl = 0; sl< NPossibles[PhenoL]; sl++)
       {
	  for (sr= 0; sr < NPossibles[PhenoR]; sr++)
	    {
	      prob = SourceTo[Possibles[PhenoL][sl]][j] 
		* ProbArete(Possibles[PhenoL][sl],
			    Possibles[PhenoR][sr],map->tr[j])
		* ToSink[Possibles[PhenoR][sr]][j+1];

	      ProbCross[NCross[Possibles[PhenoL][sl]][Possibles[PhenoR][sr]]] 
		+= prob;
	    }
       }
     
     prob = ProbCross[2]+ ProbCross[1]+ ProbCross[0];
     ProbCross[2] /= prob;
     ProbCross[1] /= prob;
     ProbCross[0] /= prob;
     expected[j] += ProbCross[2]*2 +  ProbCross[1];
   }
}

//-----------------------------------------------------------------------------
//  Prparation des structures pour la programmation dynamique de EM.
//-----------------------------------------------------------------------------
// Paramtres : 
// - la carte.
// Valeur de retour : 
//-----------------------------------------------------------------------------

void BJS_IC::PreparEM(const Carte *data)
{
    for (int i = 0; i < 4; i++)
    {
      SourceTo[i] = new double[data->NbMarqueur];
      ToSink[i] = new double[data->NbMarqueur];
    }
}

//-----------------------------------------------------------------------------
//  nettoyage des structures pour la programmation dynamique de EM.
//-----------------------------------------------------------------------------
// Paramtres :
// - la carte.
// Valeur de retour : 
//-----------------------------------------------------------------------------

void BJS_IC::NetEM(const Carte *data)
{
  for (int i = 0; i < 4; i++)
    {
      delete SourceTo[i];
      delete ToSink[i];
    }
}
