//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
// Copyright 1997 INRA (Institut National de la Recherche Agronomique).
// This file is distributed under the terms of the Q Public License version 1.0.
// 
// $Id: BioJeu.cc,v 1.38 2003/12/17 07:48:48 tschiex Exp $
//
// Description : Classe de base BioJeu.
// Divers : 
//-----------------------------------------------------------------------------

#include "BioJeu.h"

#include "Genetic.h"

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "CartaGene.h"

double BioJeu::Epsilon1 = 0.1;
double BioJeu::Epsilon2 = 0.001;

//-----------------------------------------------------------------------------
// Constructeur.
//-----------------------------------------------------------------------------

BioJeu::BioJeu()
{
  Cartage = NULL;
  Cross = Unk;
  BitJeu = 0;
  NbMarqueur = 0;
  TailleEchant = 0;
  OverallR = 0.0;
  TwoPointsFR = NULL;  
  TwoPointsDH = NULL; 
  TwoPointsLOD = NULL; 

}

//-----------------------------------------------------------------------------
// Constructeur.
//-----------------------------------------------------------------------------
// Paramtres :
// - le rfrentiel
// - le numro
// - le type
// - le nombre de marqueurs
// - le champ de bit
// Valeur de retour : 
//-----------------------------------------------------------------------------

BioJeu::BioJeu(CartaGenePtr cartag,
	       int id,
	       CrossType cross,
	       int nm, 
	       int bitjeu)
{
  Cartage = cartag;
  Id = id;
  Cross = cross;
  BitJeu = bitjeu;
  NbMarqueur = nm;
  OverallR = 0.0;

  ResetNbEMCall();
}

//-----------------------------------------------------------------------------
// Destructeur
//-----------------------------------------------------------------------------

BioJeu::~BioJeu()
{

}

//-----------------------------------------------------------------------------
// dtection des groupes, affichage et mise  jour de Cartage.
//-----------------------------------------------------------------------------
// Paramtres : 
// - disthres seuil de la distance
// - lodthres seuil du lod
// Valeur de retour : le nombre de groupes trouvs
//-----------------------------------------------------------------------------

int BioJeu::Groupe(double disthres, double lodthres) const
{
  int i, j, k, l, color, nbg;
  NodintPtr *ladj;
  NodintPtr padj, padjw;
  intPtr visited;
  NodptrPtr llink, plink, wlink;

  // initialisation de la liste d'adjacence(dimension max)
    
  ladj = new NodintPtr[Cartage->NbMarqueur + 1];
  for (i = 0; i <= Cartage->NbMarqueur; i++)
    ladj[i]=NULL;

  i = 0;
  j = 0;
  k = 0;
  l = 0;

  // remplissage de la liste d'adjacence
  while (i < Cartage->NbMS) 
    {
      if (Cartage->BitJeuMarq[Cartage->MarkSelect[k]] & BitJeu) 
	{
	  i++;
	  j = i;
	  l = k + 1;
	  while (j < Cartage->NbMS) 
	    {
	      if (Cartage->BitJeuMarq[Cartage->MarkSelect[l]] & BitJeu) 
		{
		  j++;
		  if (GroupTest(disthres, lodthres, Cartage->MarkSelect[k], Cartage->MarkSelect[l])) 
		    {
		      padj = new Nodint;
		      padj->vertex = Cartage->MarkSelect[l];
		      padj->next = ladj[Cartage->MarkSelect[k]];
		      ladj[Cartage->MarkSelect[k]] = padj;
		      
		      padj = new Nodint;
		      padj->vertex = Cartage->MarkSelect[k];
		      padj->next = ladj[Cartage->MarkSelect[l]];
		      ladj[Cartage->MarkSelect[l]] = padj;
		    }
		}
	      l++;
	    }
	}	
      k++;
    }   
  
  // parcours en profondeur
  // initialisations
  
  visited = new int[Cartage->NbMarqueur + 1];

  for (i = 0; i <= Cartage->NbMarqueur; i++)
    visited[i] = -1;

  color = 0;

  llink = NULL;

  i = 0;
  k = 0;

  while (i < Cartage->NbMS) 
    {
      if (Cartage->BitJeuMarq[Cartage->MarkSelect[k]] & BitJeu) 
	{
	  i++;
	  if (visited[Cartage->MarkSelect[k]] == -1) 
	    {
	      plink = new Nodptr;
	      plink->next = llink;
	      plink->first = NULL;
	      llink = plink;
	      ++color;
	      DFSVisit(ladj, visited, Cartage->MarkSelect[k], color, plink);	
	    };
	}
      k++;
    }
  
  // nettoyage des structures du parcours

  delete [] visited;
  delete [] ladj;

  // Au cas o

  plink = Cartage->Group;

  while (plink != NULL) {

    padj = plink->first;

    while (padj != NULL) 
      {
	
	padjw = padj;
	padj = padj->next;
	delete padjw; 
      }

    wlink = plink;
    plink = plink->next;
    delete wlink;
  }

  // Mise  jour

  Cartage->Group = llink;
  Cartage->DisThres = disthres;
  Cartage->LODThres = lodthres;
  
  // Affichage des groupes.
  
  GroupMess(disthres, lodthres);

  sprintf(bouf, "\n%9s : Marker ID List ...\n", "Group ID");
  pout(bouf);

  nbg = 0;   

  plink = llink;

  while (plink != NULL) {

    sprintf(bouf, "%9d :",++nbg);
    pout(bouf);

    padj = plink->first;

    while (padj != NULL) 
      {
	sprintf(bouf, " %d", padj->vertex);
	pout(bouf);
	
	padjw = padj;
	padj = padj->next;
      }
    
    sprintf(bouf,  "\n");
    pout(bouf);

    llink = plink;
    plink = plink->next;
  }
  
  return nbg;
  
}

//-----------------------------------------------------------------------------
// parcours en profondeur
//-----------------------------------------------------------------------------
// Paramtres : 
// - liste d'adjacence
// - vertex
// - couleure 
// Valeur de retour : 
//-----------------------------------------------------------------------------

void BioJeu::DFSVisit(NodintPtr *ladj, 
		      intPtr visited, 
		      int vertex, 
		      int color,
		      NodptrPtr plink) const
{
  NodintPtr padj, padjw;
  NodintPtr pllink;
    
  visited[vertex] = 0; // blanc pour le premier passage

  padj = ladj[vertex];

  while (padj != NULL) {
    if (visited[padj->vertex] == -1)
      DFSVisit(ladj, visited, padj->vertex, color, plink);
    padjw = padj;
    padj = padj->next;
    delete padjw;
  }

  pllink = new Nodint;
  pllink->vertex = vertex;
  pllink->next = plink->first;
  plink->first = pllink;
   
  visited[vertex] = color;
  
}

//-----------------------------------------------------------------------------
//  Calcul de la vraisemblance maximum par EM.
//-----------------------------------------------------------------------------
// Paramtres : 
// - une carte
// - seuil de convergence
// Valeur de retour : la vraisemblance maximum
//-----------------------------------------------------------------------------

double BioJeu::ComputeEM(Carte *map)
{
  int i,DomainHit = 0;
  double CoutSauv, Cout = -1e100;
  int *ordre = map->ordre;
  double * expected;

  NbEMCall++;

  // si la carte est deja finement estimee, on retourne la logv
  if (map->Converged <= Epsilon2) return map->coutEM;

  PreparEM(map);

  // + 1 = 2 cases en + pour EBreack & ERetained
  expected = new double[map->NbMarqueur + 1];

   for (i = 0; i < map->NbMarqueur + 1; i++) expected[i] = 0.0;
  
   // si la carte a deja ete estimee grossierement, on efface pas le boulot !
   if (map->Converged > Epsilon1) {
     for (i = 0; i < map->NbMarqueur - 1; i++)
       map->tr[i] = GetTwoPointsFR(ordre[i],ordre[i+1]);
     map-> ret = 0.3;
   }

   // on estime finement
  do   {
    CoutSauv = Cout;
    Cout = ComputeExpected(map, expected);
    //    printf("LogVraisemblance: %f\n",Cout);

    if (((Cout - CoutSauv) < -Close2Zero) && !DomainHit)
      NbEMUnconv++;
    
    DomainHit |= Maximize(map, expected);
  }
  while (Cout-CoutSauv > Epsilon2);
  
  if (DomainHit) NbEMHit++;

  map->Converged = Epsilon2;
  map->coutEM = Cout;
  
  delete [] expected;

  NetEM(map);

  return Cout;
}
//-----------------------------------------------------------------------------
//  Calcul de la vraisemblance maximum par EM.
//  Une interface plus sophistiquee:
//  On converge jusqu'a epsilon1. Si la vraisemblance est inferieure a
//  threshold on laisse tomber, sinon on converge a epsilon2
//-----------------------------------------------------------------------------
// Paramtres : 
// - une carte
// - niveau a partir duquel on convergera a epsilon2
// Valeur de retour : la vraisemblance maximum.
//-----------------------------------------------------------------------------

double BioJeu::ComputeEMS(Carte *map, 
			  double threshold)
{
  int i,DomainHit = 0;
  int *ordre = map->ordre;
  double CoutSauv,Cout = -1e100;
  double * expected;


  // si la carte est deja grossierement estimee et que sa logv est
  // eloignee du seuil, on retourne la logv
  if ((map->Converged <= Epsilon1) &&
      (map->coutEM < threshold))
    return map->coutEM;

  NbEMCall++;
  
  // + 1 = 2 cases en + pour EBreack & ERetained
  expected = new double[map->NbMarqueur + 1];

  for (i = 0; i < map->NbMarqueur + 1; i++) expected[i] = 0.0;

  PreparEM(map);

  // si la carte n'est pas du tout estimee, on la remplit avec des
  // valeurs de demarrage pas totalement debiles
  if (map->Converged > Epsilon1) {
    for(i = 0; i < map->NbMarqueur - 1; i++)
      map->tr[i] = GetTwoPointsFR(ordre[i],ordre[i+1]);
    map->ret = 0.3;

    // et on l'estime grossierement
    do
      {
	CoutSauv = Cout;
	Cout = ComputeExpected(map, expected);
	
	if (((Cout - CoutSauv) < -Close2Zero) && !DomainHit) 
	  NbEMUnconv++;
	
	DomainHit |= Maximize(map, expected);
      }
    while (Cout-CoutSauv > Epsilon1);
    map->Converged =Epsilon1;  
  }
  // sinon, elle a deje ete estimee grossierement et on recupere le logv
  else
    Cout =  map->coutEM;

  // si on est pres du seuil on affine l'estimation
  if (Cout >= threshold) {
    while (Cout-CoutSauv > Epsilon2)
      {
	CoutSauv = Cout;
	Cout = ComputeExpected(map, expected);
	
	if (((Cout - CoutSauv) < -Close2Zero) && !DomainHit) {
	  sprintf(bouf, "\nBug ComputeEMS 2 : DeltaV = %f\n",Cout-CoutSauv);
	  perr(bouf);
	}
	
	DomainHit |= Maximize(map, expected);
      }
    map->Converged =Epsilon2;
  }

  if (DomainHit) NbEMHit++;

  map->coutEM = Cout;

  delete [] expected;

  NetEM(map);
  
  return Cout;
}

//-----------------------------------------------------------------------------
// Etape de maximisation 
//-----------------------------------------------------------------------------
// Paramtres : 
// - la carte
// - le vecteur d'expectation
// Valeur de retour : 
//-----------------------------------------------------------------------------
int BioJeu::Maximize(Carte* map, double *expected) const
{
  int j, DomainHit = 0;
  double temp;

  int NbM = map->NbMarqueur;
   
  for (j = 0; j < map->NbMarqueur - 1; j++) {
    temp = (expected[j]) / (double)NbMeiose;
    if (temp > Em_Max_Theta) {
      temp =  Em_Max_Theta;
      DomainHit = 1;
    }
    else if (temp < Em_Min_Theta) {
      temp =  Em_Min_Theta;
      DomainHit = 1;      
    }
    
    map->tr[j] = temp;
    expected[j] = 0.0;
  }


  //verrue RH...
  
  if (EBreak > 0.0)
    { 
      // estimation de la proba de retention: nbre de retenus casses / casses
      temp = ERetained/EBreak;
      
      if (temp > EM_MAX_RETAIN)  {
	temp = EM_MAX_RETAIN;
	DomainHit = 1;
      }
      else if (temp < EM_MIN_RETAIN) {
	temp = EM_MIN_RETAIN;
	DomainHit = 1;
      }
      
      map->ret = temp;
     }
  
  
  return DomainHit;
}

//-----------------------------------------------------------------------------
// Affichage simple slectif d'une carte
// Seuls les marqueurs du gnotype sont affichs
//-----------------------------------------------------------------------------
// Paramtres : 
// - la carte
// Valeur de retour :
//----------------------------------------------------------------------------
  
void BioJeu::PrintMap(Carte *data) const
{
  char form[16];
  
  sprintf(bouf, "%4d : ", Id);
  pout(bouf);
  
  for (int i = 0; i < data->NbMarqueur; i++)  
    {
      sprintf(form,"%c%d%c%d%c ", '%', strlen(Cartage->NomMarq[data->ordre[i]]), '.', strlen(Cartage->NomMarq[data->ordre[i]]), 's');
      
      if (Cartage->BitJeuMarq[data->ordre[i]] &
	  BitJeu) {
	sprintf(bouf, form, Cartage->NomMarq[data->ordre[i]]);
	pout(bouf);
      }
      else {
	sprintf(bouf, form,"");
	pout(bouf);
      }
    }
  
   sprintf(bouf,  "\n");
   pout(bouf);
}


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Paramtres : 
// Valeur de retour :
//----------------------------------------------------------------------------
  
void BioJeu::PrintTwoPointsDist(char unit[2]) const
{
  int i,j;

  sprintf(bouf, "\nData Set Number %2d :\n", Id); 
  pout(bouf);

  sprintf(bouf,  "                 ");
  pout(bouf);
  for (i = 0; i < Cartage->NbMS; i++)
    if (Cartage->BitJeuMarq[Cartage->MarkSelect[i]] &
	BitJeu) {
      sprintf(bouf, "%5.5s ",Cartage->NomMarq[Cartage->MarkSelect[i]]);
      pout(bouf);
    }
  sprintf(bouf,  "\n");
  pout(bouf);
  fflush(stdout);
  sprintf(bouf,  "                 ");
  pout(bouf);
  for (i = 0; i < Cartage->NbMS; i++)
    if (Cartage->BitJeuMarq[Cartage->MarkSelect[i]] &
	BitJeu) {
      sprintf(bouf,  "------");
      pout(bouf);
    }
  sprintf(bouf,  "\n");
  pout(bouf);
  fflush(stdout);

  for (i = 0; i< Cartage->NbMS; i++)
    {      
      if (Cartage->BitJeuMarq[Cartage->MarkSelect[i]] &
	  BitJeu) 
	{
	  sprintf(bouf, "%15.15s |",Cartage->NomMarq[Cartage->MarkSelect[i]]);
	  pout(bouf);
	  fflush(stdout);
	  for (j = 0; j < Cartage->NbMS; j++)
	    if (Cartage->BitJeuMarq[Cartage->MarkSelect[j]] &
		BitJeu)
	      if (i == j)  {
		sprintf(bouf,  "------");
		pout(bouf);
	      }
	      else {
		sprintf(bouf, "%5.1f ", 100 * Distance(unit, GetTwoPointsFR(Cartage->MarkSelect[i], 
								     Cartage->MarkSelect[j])));
		pout(bouf);
	      }
	  sprintf(bouf,  "\n");
	  pout(bouf);
	  fflush(stdout);
	}
    }
}

//-----------------------------------------------------------------------------
// retourne une carte sous forme de liste
//-----------------------------------------------------------------------------
// Paramtres : 
// - l'unit de distance
// - la carte
// Valeur de retour :
//----------------------------------------------------------------------------
  
char*** BioJeu::GetMap(char unit[2],
		       Carte *data) const
{
  
  char* temp;
  int i, j = 0;
  double dist = 0.0;
  
  char *** resultg = new char**[2];
  resultg[1] = NULL;
  
  char ** result = new char*[(2 * data->NbMarqueur) + 3];
  
  result[(2 * data->NbMarqueur) + 2] = NULL;
  
  resultg[0] = result;

  temp = new char[8];
  sprintf(temp, 
 	  "%d", 
 	  Id);
  result[0] = temp;

  temp = new char[16];
  sprintf(temp, 
 	  "%.2f", 
 	  data->coutEM);
  result[1] = temp;
  
  j = 2;
  for (i = 0; i < data->NbMarqueur; i++)  
    {
      temp = new char[strlen(Cartage->NomMarq[data->ordre[i]])+ 1];
      sprintf(temp,  
	      Cartage->NomMarq[data->ordre[i]]);
      result[j++] = temp;
      temp = new char[8];

      if (i >= 1)
	dist = dist + Distance(unit, data->tr[i-1])*100;
      else
	dist = 0.0;
      sprintf(temp, "%.1f", dist);
      result[j++] = temp;
    }

  
  return resultg;
}


//-----------------------------------------------------------------------------
// Affichage dtaill d'une carte
//-----------------------------------------------------------------------------
// Paramtres : 
// - la carte
// - le sens
// - la carte de rfrence
// Valeur de retour :
//----------------------------------------------------------------------------
  
void BioJeu::PrintDMap(Carte *data, int envers, Carte *dataref)
{

  int i,j,k,itr,suivant;
  unsigned int maxl;
  double d, totald = 0.0;
  double dk, totaldk = 0.0;
  char form[128];
  int *pos;

  pos = new int[data->NbMarqueur];

  // rcupration de la taille la plus grande d'un nom de locus.
  
  maxl = 0;
  for (i = 0; i < data->NbMarqueur; i++) {
    pos[i] = i + 1;
    j = Cartage->Represents[data->ordre[i]];
    while (j != 0) {
      if (strlen(Cartage->NomMarq[j]) > maxl)
	maxl = strlen(Cartage->NomMarq[j]);
	j = Cartage->Represents[j];
    }
    if (strlen(Cartage->NomMarq[data->ordre[i]]) > maxl)
      maxl = strlen(Cartage->NomMarq[data->ordre[i]]);
  }

  // cas de la fusion sur l'ordre, dtermination de la position absolue
 
  if (data != dataref) { 
    // recherche pour chaque marqueur de sa position
    for (i = 0; i < data->NbMarqueur; i++) {
      j = 0;
      while ( data->ordre[i] != dataref->ordre[j])
	j++;
      if (envers) 
	pos[data->NbMarqueur - i - 1] = dataref->NbMarqueur - j;
      else
	pos[i] = j + 1;
    }
  }

  sprintf(bouf, "\nData Set Number %2d :\n", Id);
  pout(bouf);
  sprintf(bouf,  "\n");
  pout(bouf);

  if (HasRH()) {
    sprintf(form, "       Markers %c%d%cDistance      Cumulative   Theta        2pt\n", '%', maxl +4, 's');
    sprintf(bouf, form, " ");
    pout(bouf);
    
    sprintf(form, " Pos  Id %c%d%c                                   (%%sage)       LOD\n\n",'%', maxl, 's');
    sprintf(bouf, form, "name","%%");
    pout(bouf);
  }
  else {
    sprintf(form, "      Markers%c%d%cDistance    Cumulative  Distance   Theta       2pt\n", '%', maxl+4, 's');
    sprintf(bouf, form, " ");
    pout(bouf);
    
    sprintf(form, "Pos  Id %c%d%c         Haldane     Haldane     Kosambi    (%%sage)      LOD\n\n",'%', maxl, 's');
    sprintf(bouf, form, "name","%%");
    pout(bouf);
  }

      for (k = 0; k < data->NbMarqueur; k++) {

	if (envers) {
	  i = data->NbMarqueur - k - 1;
	  itr = i - 1;
	  suivant = itr;
	} else { 
	  i = k;
	  itr = i;
	  suivant = i +1;
	}

	j = Cartage->Represents[data->ordre[i]];
	while (j != 0) {
	  sprintf(form, "--- %c%d%c\n",'%', maxl, 's');
	  sprintf(bouf, form, Cartage->NomMarq[j]);
	  pout(bouf);
	  
	  j = Cartage->Represents[j];
	}
    
    sprintf(form, "%s %c%d%c ","%3d %3d",'%', maxl, 's');
    sprintf(bouf, form, pos[k], data->ordre[i], 
	   Cartage->NomMarq[data->ordre[i]]);
    pout(bouf);
    
    if (k < data->NbMarqueur-1) {
      if (HasRH())  {
	d = Theta2Ray(data->tr[itr])*100;
	totald += d;	      
	sprintf(bouf, "         %6.1f cR    %7.1f cR    %5.1f %%%%     %5.1f\n", 
	       d, totald, data->tr[itr]*100,
	       GetTwoPointsLOD(data->ordre[i],
			       data->ordre[suivant]));
	pout(bouf);
      }
      else {
	d = Haldane(data->tr[itr])*100;
	dk = Kosambi(data->tr[itr])*100;
	totaldk += dk;
	totald += d;
	
	sprintf(bouf, "       %6.1f cM   %6.1f cM   %6.1f cM   %5.1f %%%%   %5.1f\n", 
	       d,
	       totald,
	       dk, 
	       data->tr[itr]*100,
	       GetTwoPointsLOD(data->ordre[i],
			       data->ordre[suivant]));
	pout(bouf);
      }
    }
      }

      if (HasRH()) {
	sprintf(bouf,  "         ---------\n");
	pout(bouf);
	sprintf(form, "             %c%d%c     %s\n",'%', maxl, 's',"%6.1f cR\n");
	sprintf(bouf, form, " ", totald); 
	pout(bouf);
	sprintf(bouf, "\n       %2d markers, log10-likelihood = %8.2f",
	       data->NbMarqueur, 
	       (data->coutEM));
	pout(bouf);
	sprintf(bouf, "\n                   log-e-likelihood = %8.2f",
	       (data->coutEM*log(10)));
	pout(bouf);
	sprintf(bouf, "\n                   retention proba. = %8.2f",
	       data->ret);
	pout(bouf);
      }
      else {
	sprintf(bouf,  "       ----------              ----------\n");
	pout(bouf);
      
	sprintf(form, "        %c%d%c        %s\n",'%', maxl, 's',"%6.1f cM               %6.1f cM\n");
	sprintf(bouf, form, " ", totald, totaldk);
	pout(bouf);
  
	sprintf(bouf, "\n       %2d markers, log10-likelihood = %8.2f",
	       data->NbMarqueur, (data->coutEM));
	pout(bouf);

	sprintf(bouf, "\n                   log-e-likelihood = %8.2f",
	       (data->coutEM*log(10)));
	pout(bouf);
      }
      sprintf(bouf,  "\n");
      pout(bouf);

      delete [] pos;
}


int BioJeu::Couplex(int m1, int m2) const {
  return (Cartage->BitJeuMarq[m1] & BitJeu &&
	  Cartage->BitJeuMarq[m2] & BitJeu);
};

