//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
// Copyright 1997 INRA (Institut National de la Recherche Agronomique).
// This file is distributed under the terms of the Q Public License version 1.0.
// 
// $Id: BioJeuSingle.h,v 1.19 2003/12/19 15:12:16 degivry Exp $
//
// Description : Prototype de la classe de base BioJeuSingle.
// Divers : 
//-----------------------------------------------------------------------------

#ifndef _BIOJEUSINGLE_H
#define _BIOJEUSINGLE_H

#include "CGtypes.h"

#include "BioJeu.h"
#include "CartaGene.h"

/** Classe des jeux de donnes simples. */
class BioJeuSingle : public BioJeu {
 public:

  /** Les observations. */
  Obs **Echantillon;

  /** Chemin+nom fichier donnees  */
  charPtr NomJeu;


  /** Constructeur. */
  BioJeuSingle();

  /** Constructeur.
      @param cartag accs  l'ensemble des informations du systme. 
      @param id le numro du jeu.
      @param nomjeu nom du jeu
      @param cross type de jeu de donne.
      @param nm nombre de marqueurs.
      @param taille de l'chantillon
      @param bitjeu champ de bit du jeu de donnes.
      @param indmarq le vecteur d'indice de marqueur
      @param echantil la matrice des informations
  */
  BioJeuSingle(CartaGenePtr cartag,
	       int id,
	       charPtr nomjeu,
	       CrossType cross,
	       int nm, 
	       int taille,
	       int bitjeu,
	       int *indmarq,
	       Obs **echantil);

  /** Destructeur. */
  virtual ~BioJeuSingle();

  /** Retourne un champ des jeux fusionns sur l'ordre(!=CON).
      @return un champ de bits
  */
  inline int BioJeuSingle::GetBJMO() const
    {
      return BitJeu;
    };
  
  /** Accs  une valeur d'un chantillon.
      @param numarq le numro du marqueur
      @param numind le numro de l'individu
      @return l'information de l'individu au marqueur
  */
  inline enum Obs BioJeuSingle::GetEch(int numarq, int numind) const
    {
      return Echantillon[IndMarq[numarq * ((Cartage->BitJeuMarq[numarq] & BitJeu) > 0)]][numind];
    }
  
  /** Compatibilite entre deux echantillons d'un marqueur
      @param numarq1 le numero du marqueur 1
      @param numarq2 le numero du marqueur 2
  */
  virtual int Compatible(int numarq1, int numarq2) const = 0;

   /** Fusion de deux marqueurs pour la creation d'un haplotype. Le
      marqueur 1 est utilise pour stocker le resultat.
      @param numarq1 marqueur 1 
      @param numarq2 marqueur 2 
      @return nothing.  
   */
  virtual void Merge(int marq1,int marq2) const = 0;
  
   /** Affichage de l'chantillon pour un marqueur
      @param numarq le numro du marqueur
  */

  virtual void DumpEchMarq(int numarq) const;
  
  /** Affichage de l'chantillon complet. */
  virtual void DumpEch(void) const;

  /** Affichage de la matrice des LOD scores 2pt. */
  virtual void DumpTwoPointsLOD() const;

  /** Affichage de la matrice des FR 2pt. */
  void DumpTwoPointsFR() const;

  /** Test de l'existance d'un RH dans l'arborence
      @return 0 ou 1
  */
  int HasRH() const {return(Cross == RH || Cross == RHD);};

  /** Test if a RISib or RISelf dataset exist in the tree.
      It returns either the CrossType or Unk(used as logical False) 
  */
  CrossType HasRI() const {
    if (Cross == RISelf || Cross == RISib) 
      return(Cross);
    else
      return(Unk);
  };
 
  double ContribOCB(int m1, int m2);  
  double NormContribOCB(int m1, int m2);  

 protected:

  /** Calcul des valeurs deux points et remplissage des matrices.*/
  virtual void ComputeTwoPoints(void);
  
  /** Accs  une valeur LOD score 2pt.
      @param m1 le numro du premier marqueur
      @param m2 le numro du second marqueur
      @return le lod
  */
  virtual double GetTwoPointsLOD(int m1, int m2) const;
  
  /** Accs  la distance 2pt.
      @param m1 le numro du premier marqueur
      @param m2 le numro du second marqueur
      @return la distance
  */
  virtual double GetTwoPointsDH(int m1, int m2) const;
  
  /** Accs  la fraction de recombinaison 2pt.
      @param m1 le numro du premier marqueur
      @param m2 le numro du second marqueur
      @return la fraction de recombinaison
  */
  virtual double GetTwoPointsFR(int m1, int m2) const;
  
};

#endif /* _BIOJEUSINGLE_H */
