//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
// Copyright 1997 INRA (Institut National de la Recherche Agronomique).
// This file is distributed under the terms of the Q Public License version 1.0.
// 
// $Id: Greedy.cc,v 1.34 2003/11/24 09:22:14 degivry Exp $
//
// Description : Greedy (le glouton).
// Divers :
//-----------------------------------------------------------------------------


//  -------------------------------------------------------------------
//    Une tentative de Greedy local search pour le BC
//    -------------------------------------------------------------------

//  quelques idees:
//    - random walk
//    - ajouter hash table separee (plus grande) pour detection des boucles
//    car on peut rester a tourner sur un plateau eloigne de l'optimum
//    puisque seules les N meilleures sont conservees
//    - ajouter autre voisinage
//    - ne pas explorer tout le voisinage, surtout au debut
//    - faire du random biased sampling
//    - faire de l'exploration en parallele au debut (style AG), puis
//    focalisation par suppression de traces communes
//    - ajouter de nouvelles heuristiques


//    - danger: si on ajoute 3 en cas de saut, il faut inhiber la
//    possibilite de resauter pdt >3 ou risque de non terminaison !
   
//    Plus tard:
   
//    rajouter un voisinage base sur la decomposition en cartes intra
//    consistant a retourner l'ensemble des marqueurs d'une des cartes
//    (si faible nombre de communs) dans la carte globale

//    traiter en simultane intra et extra

#include <limits.h>
#include <stdlib.h>
#include <search.h>
#include "CartaGene.h"
#include "BioJeu.h"
#ifdef CGBENCH
int insertPP(Tas *tas, double lastbestcost, Carte *map, int iter);
#endif

#include <stdio.h>
//#include <assert.h>

//-----------------------------------------------------------------------------
// Lancement du glouton.
//-----------------------------------------------------------------------------
// Paramtres : 
// - 
// - 
// Valeur de retour : 
//-----------------------------------------------------------------------------

void CartaGene::Greedy(int NR, 
		       int NI,
		       int TMin, 
		       int TMax,
		       int Ratio)
{
  if (Ratio < 0 || Ratio > 100)
    {
      sprintf(bouf,  "Error : value expected for Ratio in [0,100].\n");
      perr(bouf);
      return;
    }
  // Note: GreedyNew(...,100) != GreedyOld(...) car GreedyNew arrete exploration
  //       du voisinage desque meilleure solution trouvee
  if (Ratio == 0) GreedyOld(NR, NI, TMin, TMax);
  else GreedyNew(NR, NI, TMin, TMax, Ratio);
}

void CartaGene::GreedyOld(int NR, 
			  int NI,
			  int TMin, 
			  int TMax)
{

  // quelques vrifications 

  if (Heap.HeapSize == 0) 
    {
      sprintf(bouf,  "Error : Empty heap.\n");
      perr(bouf);
      return;
    }
  
  if (NR < 1)
    {
      sprintf(bouf,  "Error : value expected for NR : > 0.\n");
      perr(bouf);
      return;
    }

  //  if (NI < 0)    {
  //      sprintf(bouf,  "Error : value expected for NI : >= 0.\n");
  //      perr(bouf);
  //      return;   }

  if ((TMin < 0) || (TMin > TMax))
    {
      sprintf(bouf,  "Error : value expected for Tmin : 0 =< & < Tmax.\n");
      perr(bouf);
      return;
    } 
  
  if ((TMax < 1) || (TMax < TMin))
    {
      sprintf(bouf,  "Error : value expected for Tmax : 0 =<  & > Tmin.\n");
      perr(bouf);
      return;
    } 
  // fin des vrifications

  int NbRun;
  int NbIter;
  int TabooMin, TabooMax;
  double finalcost;
  double cost, localbest;
  int Taboolen;
  int TabooLenMin;
  int TabooLenMax;
  Carte FinalMap(this, NbMS, MarkSelect);
  Carte TheMap(this, NbMS, MarkSelect);
  Carte TempMap(this, NbMS, MarkSelect);
  Carte *InitMap;
  int iter;
  int beste1, beste2;
  // bwlen initialisee car pourrait passer dans test sans avoir recue de valeur
  int e1, e2, wlen, bwlen = 0;
  int i, j;
  int Improve;
  int NeighSize;
  int StarMax = NbMS;
  intPtr *Age;

  if (NI < 0) NbIter = -NI;
  else NbIter = NI+2*NbMS;

  NbRun = NR;

  iter = 0;

  TabooLenMin = TMin;
  TabooLenMax = TMax;
  NeighSize = ((NbMS-2)*(NbMS-1))/2;
  TabooMin =(TabooLenMin*NeighSize)/100;
  TabooMax =(TabooLenMax*NeighSize)/100;

  Age = new intPtr[NbMS-1];
  for (i=0; i<NbMS-1; i++) Age[i] = new int [NbMS+1];

  InitMap = Heap.Best();
  InitMap->CopyFMap(&TheMap);

  finalcost = ComputeEM(&TheMap);
  TheMap.CopyFMap(&FinalMap);
#ifdef CGBENCH
  insertPP(&Heap, finalcost, &TheMap, iter);
#else
  Heap.Insert(&TheMap, iter);
#endif
  if (!QuietFlag) PrintMap(&TheMap);
  
  /* Run loop */
  for (j = 0; j< NbRun; j++) {
    if (!QuietFlag) {
      sprintf(bouf, "Run number %d\n", j);
      pout(bouf);
    }
    fflush(stdout);

    for (e1 = 0; e1 < NbMS-1; e1++)
      for (e2 = e1 + 2; e2 <= NbMS; e2++)
	Age[e1][e2] = INT_MIN;
    /* Iteration loop */
    for (i = 0; i< NbIter; i++) {
      /* Greedy Step */
      localbest = -1e100;
      beste1 = beste2 = 0;
      Improve = 0;
      Taboolen = randomax(TabooMax-TabooMin+1)+TabooMin;
      for (e1 = 0; e1 < NbMS-1; e1++)
	for (e2 = e1 + 2; e2 <= NbMS-(e1==0); e2++)
        {
	  TheMap.CopyMap(&TempMap);
	  TempMap.Apply2Change(e1, e2);

	  cost = ComputeEMS(&TempMap, finalcost-2.0);
	  
#ifdef CGBENCH
	  wlen = insertPP(&Heap, finalcost, &TempMap, iter);
#else
	  wlen = Heap.Insert(&TempMap, iter);
#endif

	  if (VerboseFlag>1) {
	    sprintf(bouf, "2-change: (%d,%d), Wlen: %d, DLogLike: %f\n",
		   e1, e2, wlen, finalcost-cost);
	    pout(bouf);
	  }
	  fflush(stdout);
	  /* localbest est le meilleur cout possible pour l'itration
	     Ca peut tre un cout de Taboo si aspiration */
	  
	  if (cost > finalcost+0.000001)
          {
	    localbest = finalcost = cost;
	    beste1 = e1;
	    beste2 = e2;
	    Improve = 1;
	    TempMap.CopyFMap(&FinalMap);
	  } else if ((cost > localbest) && (Age[e1][e2] <  i - Taboolen)) {
	    localbest = cost;
	    beste1 = e1;
	    beste2 = e2;
	    bwlen = wlen;
	  }
	  
	  // traitement de la demande d'arrt.
	  if ( StopFlag )
	    { 
	      StopFlag = 0;
	      
	      // libration
	      
	      for (i=0; i<NbMS-1; i++) delete [] Age[i];
	      delete [] Age;
	      
	      // 
	      sprintf(bouf,  "Aborted!\n");
	      pout(bouf);
	      fflush(stdout);
	      return;
	    }
	}
      if (bwlen) {
	TheMap.BuildNewMapL();
	localbest = ComputeEMS(&TheMap, finalcost-2.0);
#ifdef CGBENCH
	insertPP(&Heap, finalcost, &TheMap, iter);
#else
	Heap.Insert(&TheMap, iter);
#endif
	Improve = 2;
      } else {
	if (VerboseFlag>1) {
          sprintf(bouf, "2-change applied: (%d,%d)\n", beste1, beste2);
	  pout(bouf);
	}
	TheMap.Apply2Change(beste1, beste2);
	Age[beste1][beste2] = i;
      }

      fflush(stdout);
      if (!QuietFlag) switch (Improve)
	{
	case 0:
	  sprintf(bouf,  "-");
	  pout(bouf);
	  break;
	case 1:
	  sprintf(bouf, "\n[%3.2f]",finalcost);
	  pout(bouf);
	  break;
	case 2:
	  sprintf(bouf,  "*");
	  pout(bouf);
	  StarMax--;
	  if (StarMax <= 0) Improve = 0;
	  break;
	}
      fflush(stdout);

      NbIter += Improve;
      iter++;
      if (VerboseFlag)  {
	sprintf(bouf, "(%3.1f) ",(localbest-finalcost));
	pout(bouf);
	PrintMap(&TheMap);
      }
      fflush(stdout);
    }
    
    TheMap.BuildNewMapL();
    cost = ComputeEMS(&TheMap, finalcost-2.0);
#ifdef CGBENCH
    insertPP(&Heap, finalcost, &TheMap, iter);
#else
    Heap.Insert(&TheMap, iter);
#endif
    if (cost > finalcost) {
      finalcost = cost;
      TheMap.CopyFMap(&FinalMap);
    }

  }

  for (i=0; i<NbMS-1; i++) delete [] Age[i];
  delete [] Age;

  sprintf(bouf,  "\n");
  pout(bouf);
}

/* ************************************************************************ */
/* ************************************************************************ */

#define EPSILON 0.01

typedef int (*cmp_t)(const void *, const void *);

int HeuristicLOD = 1; // = 1 : HeuristicDist = LOD 2pts , = 0 : HeuristicDist = F.Rec. 
double HeuristicDist[MAXNBMARKERS];
double HeuristicDistWorst = -1.0;
double HeuristicDistBest = 1e100;
int NbMrk = -1; // = NbMS // car NbMS inconnu dans la fonction trpair

inline int min(int a, int b)
{
  return ((a<b)?a:b);
}
inline int max(int a, int b)
{
  return ((a>b)?a:b);
}

inline double min(double a, double b)
{
  return ((a<b)?a:b);
}
inline double max(double a, double b)
{
  return ((a>b)?a:b);
}

inline double meandist(int m1, int m2)
{
  double trm1 = ((m1 == 0)?HeuristicDistWorst:HeuristicDist[m1-1]);
  double trm2 = ((m2 < NbMrk)?HeuristicDist[m2-1]:HeuristicDistWorst);
  return (trm1 + trm2) / 2.;
}

int cmpMrkPairsMean(int *p1, int *p2)
{
  double trp1 = meandist(p1[0], p1[1]);
  double trp2 = meandist(p2[0], p2[1]);

  return ((trp1 > trp2)?-1:((trp1 == trp2)?0:1));
}

int cmpMrkPairsMin(int *p1, int *p2)
{
  int m1_1 = p1[0];
  int m1_2 = p1[1];
  double tr1_1min = ((m1_1 == 0)?HeuristicDistBest:HeuristicDist[m1_1-1]);
  double tr1_2min = ((m1_2 < NbMrk)?HeuristicDist[m1_2-1]:HeuristicDistBest);
  double tr1_1max = ((m1_1 == 0)?HeuristicDistWorst:HeuristicDist[m1_1-1]);
  double tr1_2max = ((m1_2 < NbMrk)?HeuristicDist[m1_2-1]:HeuristicDistWorst);
  
  int m2_1 = p2[0];
  int m2_2 = p2[1];
  double tr2_1min = ((m2_1 == 0)?HeuristicDistBest:HeuristicDist[m2_1-1]);
  double tr2_2min = ((m2_2 < NbMrk)?HeuristicDist[m2_2-1]:HeuristicDistBest);
  double tr2_1max = ((m2_1 == 0)?HeuristicDistWorst:HeuristicDist[m2_1-1]);
  double tr2_2max = ((m2_2 < NbMrk)?HeuristicDist[m2_2-1]:HeuristicDistWorst);
  
  if (min(tr1_1min, tr1_2min) < min(tr2_1min, tr2_2min) + EPSILON)
    return -1;
  else if (min(tr1_1min, tr1_2min) > min(tr2_1min, tr2_2min) - EPSILON)
    return 1;
  else {
    double tr1;
    double tr2;
    if (tr1_1min < tr1_2min) tr1 = tr1_2max;
    else tr1 = tr1_1max;
    if (tr2_1min < tr2_2min) tr2 = tr2_2max;
    else tr2 = tr2_1max;
    return ((tr1 < tr2)?-1:((tr1 == tr2)?0:1));
  }
}

int cmpMrkPairsMax(int *p1, int *p2)
{
  int m1_1 = p1[0];
  int m1_2 = p1[1];
  double tr1_1min = ((m1_1 == 0)?HeuristicDistBest:HeuristicDist[m1_1-1]);
  double tr1_2min = ((m1_2 < NbMrk)?HeuristicDist[m1_2-1]:HeuristicDistBest);
  double tr1_1max = ((m1_1 == 0)?HeuristicDistWorst:HeuristicDist[m1_1-1]);
  double tr1_2max = ((m1_2 < NbMrk)?HeuristicDist[m1_2-1]:HeuristicDistWorst);
  
  int m2_1 = p2[0];
  int m2_2 = p2[1];
  double tr2_1min = ((m2_1 == 0)?HeuristicDistBest:HeuristicDist[m2_1-1]);
  double tr2_2min = ((m2_2 < NbMrk)?HeuristicDist[m2_2-1]:HeuristicDistBest);
  double tr2_1max = ((m2_1 == 0)?HeuristicDistWorst:HeuristicDist[m2_1-1]);
  double tr2_2max = ((m2_2 < NbMrk)?HeuristicDist[m2_2-1]:HeuristicDistWorst);
  
  if (max(tr1_1min, tr1_2min) > max(tr2_1min, tr2_2min) + EPSILON)
    return -1;
  else if (max(tr1_1min, tr1_2min) < max(tr2_1min, tr2_2min) - EPSILON)
    return 1;
  else {
    double tr1;
    double tr2;
    if (tr1_1min > tr1_2min) tr1 = tr1_2max;
    else tr1 = tr1_1max;
    if (tr2_1min > tr2_2min) tr2 = tr2_2max;
    else tr2 = tr2_1max;
    return ((tr1 > tr2)?-1:((tr1 == tr2)?0:1));
  }
}

void CartaGene::GreedyNew(int NR, 
			  int NI,
			  int TMin, 
			  int TMax,
			  int Ratio)
{

  // quelques vrifications 

  if (Heap.HeapSize == 0) 
    {
      sprintf(bouf,  "Error : Empty heap.\n");
      perr(bouf);
      return;
    }
  
  if (NR < 1)
    {
      sprintf(bouf,  "Error : value expected for NR : > 0.\n");
      perr(bouf);
      return;
    }

  //  if (NI < 0)    {
  //      sprintf(bouf,  "Error : value expected for NI : >= 0.\n");
  //      perr(bouf);
  //      return;   }

  if ((TMin < 0) || (TMin > TMax))
    {
      sprintf(bouf,  "Error : value expected for Tmin : 0 =< & < Tmax.\n");
      perr(bouf);
      return;
    } 
  
  if ((TMax < 1) || (TMax < TMin))
    {
      sprintf(bouf,  "Error : value expected for Tmax : 0 =<  & > Tmin.\n");
      perr(bouf);
      return;
    } 
  // fin des vrifications

  // tableau pour trie de tout le voisinage avant retenue d'une partie seulement
  int *sortedmrk = new int[(NbMS+1) * (NbMS+1)];
  int bestposition; // uniquement pour affichage

  int NbRun;
  int NbIter;
  int TabooMin, TabooMax;
  double finalcost;
  double cost, localbest;
  int Taboolen;
  int TabooLenMin;
  int TabooLenMax;
  Carte FinalMap(this, NbMS, MarkSelect);
  Carte TheMap(this, NbMS, MarkSelect);
  Carte TempMap(this, NbMS, MarkSelect);
  Carte *InitMap;
  int iter;
  int beste1, beste2;
  // bwlen initialisee car pourrait passer dans test sans avoir recue de valeur
  int e1, e2, wlen, bwlen = 0;
  int i, j;
  int Improve;
  int NeighSize;
  int StarMax = NbMS;
  intPtr *Age;

  if (NI < 0) NbIter = -NI;
  else NbIter = NI+2*NbMS;

  NbRun = NR;

  iter = 0;

  TabooLenMin = TMin;
  TabooLenMax = TMax;
  NeighSize = (int)((double) Ratio * (((NbMS-2)*(NbMS-1))/2) / 100.);
  TabooMin =(TabooLenMin*NeighSize)/100;
  TabooMax =(TabooLenMax*NeighSize)/100;

  Age = new intPtr[NbMS-1];
  for (i=0; i<NbMS-1; i++) Age[i] = new int [NbMS+1];

  InitMap = Heap.Best();
  InitMap->CopyFMap(&TheMap);

  finalcost = ComputeEM(&TheMap);
  TheMap.CopyFMap(&FinalMap);
#ifdef CGBENCH
  insertPP(&Heap, finalcost, &TheMap, iter);
#else
  Heap.Insert(&TheMap, iter);
#endif
  if (ArbreJeu->Cross == Mor) {
    HeuristicLOD = 1;
    HeuristicDistWorst = -1.;
    HeuristicDistBest = 1e100;
  } else {
    HeuristicLOD = 0;
    HeuristicDistWorst = 1.;
    HeuristicDistBest = 0.;
  }
  for (int idx=0; idx<NbMS-1; idx++) {
    if (HeuristicLOD)
      HeuristicDist[idx] = GetTwoPointsLOD(TheMap.ordre[idx], TheMap.ordre[idx+1]);
    else
      HeuristicDist[idx] = TheMap.tr[idx];
  }
  if (!QuietFlag) PrintMap(&TheMap);
  
  NbMrk = NbMS;
#ifdef CGDEBUG
  printf("--- Neighborhood Size Ratio = %d\n", Ratio);
#endif

  /* Run loop */
  for (j = 0; j< NbRun; j++) {
    if (!QuietFlag) {
      sprintf(bouf, "Run number %d\n", j);
      pout(bouf);
    }
    fflush(stdout);

    for (e1 = 0; e1 < NbMS-1; e1++)
      for (e2 = e1 + 2; e2 <= NbMS; e2++)
	Age[e1][e2] = INT_MIN;
    /* Iteration loop */
    for (i = 0; i< NbIter; i++) {
      /* Greedy Step */
      localbest = -1e100;
      beste1 = beste2 = 0;
      Improve = 0;
      Taboolen = randomax(TabooMax-TabooMin+1)+TabooMin;

      int nbpairs = 0;
      for (e1 = 0; e1 < NbMS-1; e1++) {
	for (e2 = e1 + 2; e2 <= NbMS-(e1==0); e2++) {
	  sortedmrk[nbpairs * 2] = e1;
	  sortedmrk[nbpairs * 2 + 1] = e2;
	  nbpairs++;
	}
      }
      qsort(sortedmrk, nbpairs, sizeof(int) * 2, (cmp_t) ((HeuristicLOD)?cmpMrkPairsMin:cmpMrkPairsMax));
#ifdef CGDEBUG
      printf("\n");
      for (int index=0; index<NbMS; index++) {
	printf(" %4d", TheMap.ordre[index]);
      } 
      printf("\n");
      for (int index=0; index<=NbMS; index++) {
	printf(" %4d", index);
      } 
      printf("\n");
      for (int index=0; index<=NbMS; index++) {
	printf(" %1.2f", (index == 0 || index == NbMS)?HeuristicDistWorst:HeuristicDist[index-1]);
      } 
      printf("\n");
      for (int index=0; index<nbpairs; index++) {
	printf("%.3f,%d,%d ", meandist(sortedmrk[index * 2], sortedmrk[index * 2 + 1]), sortedmrk[index * 2], sortedmrk[index * 2 + 1]);
      }
      printf("\n");
#endif
      // au minimum, examen des permutations des extremites
      int nbpairselected = min(nbpairs, max(NbMS * 2, (int) (((double) Ratio * nbpairs) / 100.)));
#ifdef CGDEBUG
      printf("\nnbpairselected = %d (nbpairs= %d,Ratio= %d)\n", nbpairselected, nbpairs, Ratio);
#endif
      for (int idpair = 0; idpair < nbpairselected; idpair++)
        {
	  e1 = sortedmrk[idpair * 2];
	  e2 = sortedmrk[idpair * 2 + 1];

	  TheMap.CopyMap(&TempMap);
	  TempMap.Apply2Change(e1, e2);

	  cost = ComputeEMS(&TempMap, finalcost-2.0);
	  
#ifdef CGBENCH
	  wlen = insertPP(&Heap, finalcost, &TempMap, iter);
#else
	  wlen = Heap.Insert(&TempMap, iter);
#endif

	  if (VerboseFlag>1) {
	    sprintf(bouf, "2-change: (%d,%d), Wlen: %d, DLogLike: %f\n",
		   e1, e2, wlen, finalcost-cost);
	    pout(bouf);
	  }
	  fflush(stdout);
	  /* localbest est le meilleur cout possible pour l'itration
	     Ca peut tre un cout de Taboo si aspiration */
	  
	  if (cost > finalcost+0.000001)
          {
	    localbest = finalcost = cost;
	    beste1 = e1;
	    beste2 = e2;
	    bestposition = idpair;
	    for (int idx=0; idx<NbMS-1; idx++) {
	      if (HeuristicLOD)
		HeuristicDist[idx] = GetTwoPointsLOD(TempMap.ordre[idx], TempMap.ordre[idx+1]);
	      else
		HeuristicDist[idx] = TempMap.tr[idx];
	    }
	    Improve = 1;
	    TempMap.CopyFMap(&FinalMap);
	    goto Jump1;
	  } else if ((cost > localbest) && (Age[e1][e2] <  i - Taboolen)) {
	    localbest = cost;
	    beste1 = e1;
	    beste2 = e2;
	    bestposition = idpair;
	    for (int idx=0; idx<NbMS-1; idx++) {
	      if (HeuristicLOD)
		HeuristicDist[idx] = GetTwoPointsLOD(TempMap.ordre[idx], TempMap.ordre[idx+1]);
	      else
		HeuristicDist[idx] = TempMap.tr[idx];
	    }
	    bwlen = wlen;
	  }
	  
	  // traitement de la demande d'arrt.
	  if ( StopFlag )
	    { 
	      StopFlag = 0;
	      
	      // libration
	      
	      for (i=0; i<NbMS-1; i++) delete [] Age[i];
	      delete [] Age;
	      
	      // 
	      sprintf(bouf,  "Aborted!\n");
	      pout(bouf);
	      fflush(stdout);
	      return;
	    }
      }
    Jump1:
      if (bwlen) {
	TheMap.BuildNewMapL();
	localbest = ComputeEMS(&TheMap, finalcost-2.0);
#ifdef CGBENCH
	insertPP(&Heap, finalcost, &TheMap, iter);
#else
	Heap.Insert(&TheMap, iter);
#endif
	for (int idx=0; idx<NbMS-1; idx++) {
	  if (HeuristicLOD)
	    HeuristicDist[idx] = GetTwoPointsLOD(TheMap.ordre[idx], TheMap.ordre[idx+1]);
	  else
	    HeuristicDist[idx] = TheMap.tr[idx];
	}
	Improve = 2;
      } else {
	if (VerboseFlag>1) {
          sprintf(bouf, "2-change applied: (%d,%d)\n", beste1, beste2);
	  pout(bouf);
	}
#ifdef CGDEBUG
	double mindist = 1e100;
	double maxdist = -1e100;
	double meandist = 0.;
	for (int indice=0; indice<NbMS-1; indice++) {
	  meandist += HeuristicDist[indice];
	  if (HeuristicDist[indice] > maxdist) maxdist = HeuristicDist[indice];
	  if (HeuristicDist[indice] < mindist) mindist = HeuristicDist[indice];
	}	
	meandist = meandist / (NbMS-1);
	printf("HeuristicDist: %1.3f(%3d) %1.3f(%3d) min = %1.3f mean = %1.6f max = %1.3f\n", (beste1>0)?HeuristicDist[beste1-1]:HeuristicDistWorst, beste1, (beste2<NbMarqueur)?HeuristicDist[beste2-1]:HeuristicDistWorst, beste2, mindist, meandist, maxdist);

	printf("pos= %f %% Improve= %d\n", 100. * bestposition / nbpairs, Improve);
#endif
	TheMap.Apply2Change(beste1, beste2);
	Age[beste1][beste2] = i;
      }

      fflush(stdout);
      if (!QuietFlag) switch (Improve)
	{
	case 0:
	  sprintf(bouf,  "-");
	  pout(bouf);
	  break;
	case 1:
	  sprintf(bouf, "\n[%3.2f]",finalcost);
	  pout(bouf);
	  break;
	case 2:
	  sprintf(bouf,  "*");
	  pout(bouf);
	  StarMax--;
	  if (StarMax <= 0) Improve = 0;
	  break;
	}
      fflush(stdout);

      NbIter += Improve;
      iter++;
      if (VerboseFlag)  {
	sprintf(bouf, "(%3.1f) ",(localbest-finalcost));
	pout(bouf);
	PrintMap(&TheMap);
      }
      fflush(stdout);
    }
    
    TheMap.BuildNewMapL();
    cost = ComputeEMS(&TheMap, finalcost-2.0);
#ifdef CGBENCH
    insertPP(&Heap, finalcost, &TheMap, iter);
#else
    Heap.Insert(&TheMap, iter);
#endif
    for (int idx=0; idx<NbMS-1; idx++) {
      if (HeuristicLOD)
	HeuristicDist[idx] = GetTwoPointsLOD(TheMap.ordre[idx], TheMap.ordre[idx+1]);
      else
	HeuristicDist[idx] = TheMap.tr[idx];
    }
    
    if (cost > finalcost) {
      finalcost = cost;
      TheMap.CopyFMap(&FinalMap);
    }

  }

  for (i=0; i<NbMS-1; i++) delete [] Age[i];
  delete [] Age;

  delete [] sortedmrk;

  sprintf(bouf,  "\n");
  pout(bouf);
}
