//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
// Copyright 1997 INRA (Institut National de la Recherche Agronomique).
// This file is distributed under the terms of the Q Public License version 1.0.
// 
// $Id: Greedy.h,v 1.13 2002/10/08 09:43:41 tschiex Exp $
//
// Description : Prototype de la classe Greedy (le glouton).
// Divers :
//-----------------------------------------------------------------------------

#ifndef _GREEDY_H
#define _GREEDY_H

#include "GenotypeBC.h"
#include "GenotypeIC.h"
#include "GenotypeRH.h"
#include "MiniCarte.h"


///
class Greedy {
 public:
  
  /** Constructeur.
      @param GT Le gnotype sur lequel porte le traitement.
  */
  Greedy(Genotype *GT);
  
  /** Destructeur.
    */
  ~Greedy();
  
  /** Lancement de Greedy.
      La recherche de la meilleure carte est faite avec l'algorithme de Greedy.
      Durant la recherche, les cartes trouves sont stockes dans le tas et
      ranges dans l'ordre de cout dcroissant.
      
      En fin de calcul, le tas est dpil. Suivant les drapeaux utiliss,
      les cartes sont affiches. Une liste \Ref{ListeMiniCarte} est construite
       partir de certains lments (\Ref{MiniCarte}) des cartes dpiles
      puis retourne.

      ATTENTION: la liste ListeMiniCarte est construite au niveau de la
      commande \Ref{Tas}::WriteBest(). Pensez  la dtruire.
      
      @param Vflag drapeau du mode verbeux des rsultats en cours de traitement.
      @param Qflag drapeau du mode silencieux sur la progression du traitement.
      @param Pflag drapeau de raffinage.
      @param Fsize taille de la fentre.
      @param Fthres seuil du flip
      @param Fiter flip itratif ou pas
      @param NR Nombre de boucles principales.
      @param NI Nombre d'itrations.
      @param TMin TabooMin.
      @param TMax TabooMax.
      @param OM nom du fichier de sortie des rsultats.
      @param OS autre fichier de rsultats.
      @return la liste de cartes contenues dans le tas en fin de calcul.
  */
  ListeMiniCarte * Run(int Vflag = 0, 
		       int Qflag = 0, 
		       int Pflag = 0,
		       int Fsize = 3,
		       double Fthres = 3.0,
		       int Fiter = 0,
		       int NR = 1, 
		       int NI = 0,
		       int TMin = 1, 
		       int TMax = 15,
		       char *OM = "/dev/null", 
		       char *OS = "/dev/null");
  
 private:
  
  /** Le gnotype tudi.
    */
  Genotype *MatGen;
  
};

#endif /* _GREEDY_H */
