//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
//  Copyright (C) 1997 INRA (Institut National de la Recherche Agronomique).
//
// $Id: cgsh.h,v 1.41 2004/06/30 08:13:24 degivry Exp $
//
//
//    This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Module 	: cgsh
// Projet	: CartaGene
//
// Description  : Prototype de l''interface gnrale de la librairie 
// pour le langage de script
//
// Chaque fonction correspond  une commande. 
// Comme l''interface est produite automatiquement, il est IMPORTANT, 
// de respecter un certain nombre de conventions.
//
// Il est judicieux de nommer les commandes et les argiuments de faon
// homogne, sans utiliser de majuscules, ni de car, spciaux.
//
// Quelques principes de base pour nommer les commandes :
// - le nom des commandes commence par un prfix(ds pour Data Set, gr pour group ...) permettant de regrouper le plus possible les commandes en familles.
// - il est suivi d''un verbe(load, merge...)
// - enfin on peut rajouter quelques infos(idm pour ID de marqueur, nmm nom de marqueur....) permettant de rajouter un peu de smantique.
// - Penser  l''utilisateur final, trouver le compromis entre la concision et la smantique du mot.
//
// Exemple : grgetidm veut dire acquisition d''un group sous la forme d''une liste d''identifiant de marqueurs. 
//
// Quelques principes de base pour nommer les arguments :
// - Ne pas ajouter des noms d'' arguments existant dja.
// - Les majuscules sont autorises(l''utilisateur n''a pas  les saisirs)
// - Assembler les mots du gnral au prcis SetId, et nom pas IdSet.
//
// - les arguments "automatiques" :
// char help[3] est un argument OBLIGATOIRE. permettant d''implmenter -h.
// char **textout est un argument standard 
// pour retourner une chaine de caractre  Tcl.
// int **lmiout est un argument standard 
// pour retourner une liste d''indice de marqueur  Tcl.
//
// Encore un conseil :
// Il est juducieux de rajouter un minimum de commentaires 
// devant chaque fonction pour la documentation technique automatique,
// en franais.
//-----------------------------------------------------------------------------
//-INRA--------------------Station de Biomtrie et d''Intelligence Artificielle
//-----------------------------------------------------------------------------

#ifndef _INTERFACE_H
#define _INTERFACE_H

#include "CartaGene.h"
#include "tsp.h"
//#include "../calcul/CartaGene.h"

#define NBM 8192 // taille max d''une liste de marqueurs.

/** Chargement d'un jeu de donnes.
    
    @param help pour l'option -h.
    @param FileName nom de fichier contenant un jeu de donnes.
    @return textout une ligne de texte informative(out). 
*/
void dsload(char help[3], char *FileName, char **textout);

/** Cration un jeu de donnes fusionnant les deux autres sur l'ordre.

   @param help pour l'option -h.
   @param SetID1 identifiant d'un jeu existant.
   @param SetID2 identifiant d'un jeu existant.
   @param textout une ligne de texte informative(out).
*/
void dsmergor(char help[3], int SetID1, int SetID2, char **textout);


/** Cration un jeu de donnes fusionnant les deux autres gntiquement.

   @param help pour l'option -h.
   @param SetID1 identifiant d'un jeu existant.
   @param SetID2 identifiant d'un jeu existant.
   @param textout une ligne de texte informative(out).
*/
void dsmergen(char help[3], int SetID1, int SetID2, char **textout);

/** Affichage d'un rsum sur tous les jeux de donnes.
    
    @param help pour l'option -h
*/
void dsinfo(char help[3]);

/** retourne une liste des rsums de tous les jeux de donnes.
    
    @param help pour l'option -h
*/
void dsget(char help[3], char ***ltextout);

/** Affichage de tous les marqueurs et des BioJeus auquels ils appartiennent.
    
    @param help pour l'option -h
*/
void mrkinfo(char help[3]);

/** pour positionner la slection des marqueurs

    @param help pour l'option -h.
    @param MrkList vecteur de marqueurs.
  */
void mrkselset(char help[3], int MrkList[NBM]);

/** pour rcuprer la slection des marqueurs

    @param help pour l'option -h.
    @param lmiout une liste d'id de marqueurs(out).
  */
void mrkselget(char help[3], int **lmiout);

/** pour rcuprer la slection des marqueurs

    @param help pour l'option -h.
    @param lmiout une liste d'id de marqueurs(out).
  */
void mrkallget(char help[3], int **lmiout);

/** Affichage de la matrice LOD 2 points pour la slection
    
    @param help pour l'option -h
*/
void mrklod2p(char help[3]);

/** Affichage des marqueurs doubles
    
    @param help pour l'option -h
*/
void mrkdouble(char help[3]);

/** Affichage de la matrice distance  2 points pour la slection
    
    @param help pour l'option -h
*/
void mrkdist2p(char help[3], char UnitFlag[2]);

/** Affichage de la matrice FR  2 points pour la slection
    
    @param help pour l'option -h
*/
void mrkfr2p(char help[3]);

/** Fusion de deux marqueurs compatibles

   @param help pour l'option -h.
*/

void mrkmerge(char help[3], int MrkID1, int MrkId2);

/** To retrieve the merged Loci

   @param help pour l'option -h.
*/
void mrkmerget(char help[3], char ***ltextout);


/** Convertion d'un identifiant numrique en identifiant textuel.

   @param help pour l'option -h.
   @param MrkID identifiant d'un locus existant.
   @param textout un nom de marqueur, ou une chaine vide.
*/
void mrkname(char help[3], int MrkID, char **textout);

/** Et rciproquement.

   @param help pour l'option -h.
   @param MrkName identifiant d'un locus existant.
   @return un numro de marqueur ou -1.
*/
int mrkid(char help[3], char *MrkName);

/** Le numro du dernier marqueur
    .
    @param help pour l'option -h.
    @return un numro de marqueur.
*/
int mrklast(char help[3]);

/** Affectation de la verbosit

    @param help pour l'option -h.
    @param Flag le niveau
*/
void verbset(char help[3], int Flag);

/** Affectation du silencieux

    @param help pour l'option -h.
    @param Flag le niveau
*/
void quietset(char help[3], int Flag);


/** pour faire un simple EM sur l'ordre de la slection

    @param help pour l'option -h.
*/
void sem(char help[3]);

/** heuristique base sur le critre de distance

    @param help pour l'option -h.
*/
void nicemapd(char help[3]);

/** heuristique base sur le critre de distance

    @param help pour l'option -h.
*/
void mfmapd(char help[3]);

/** heuristique base sur le critre de LOD

    @param help pour l'option -h.
*/
void nicemapl(char help[3]);

/** heuristique base sur le critre de LOD

    @param help pour l'option -h.
*/
void mfmapl(char help[3]);

/** Build
 
    @param NbMap le nombre de cartes construites simultanments
*/
void build(char help[3],
	   int NbMap);

/** Build
    
    @param AddThres minimum ncessaire  l'acceptation de l'insertion d'un marqueur
    @param keepThres Seuil  l'intrieur duquel les ordres sont conservs.
    @param MrkList vecteur de marqueurs.
    @param MrkList entier qui indique que l'on veut tester les autres marqueurs 0 pas test, 1 construction et test, 2 test
*/
void buildfw(char help[3],
	     double KeepThres, 
	     double AddThres,
	     int MrkList[NBM],
	     int Flag);

/** Recuit Simul
    La recherche de la meilleure carte est faite avec l'algorithme de Recuit.
    Durant la recherche, les cartes trouves sont stockes dans le tas et
    ranges dans l'ordre de cout dcroissant.
    @param NbTries Nombre d'essai par temprature.
    @param InitTemp Temprature de dbut.
    @param FinalTemp Temprature finale.
    @param Cooling Vitesse de refroidissement.
*/
void annealing(char help[3],
	       int NbTries,
	       double InitTemp,
	       double FinalTemp,
	       double Cooling);

/** Lancement de Greedy.
    @param NbLoop Nombre de boucles principales.
    @param NbIter Nombre d'itrations.
    @param TabooMin TabooMin.
    @param TabooMax TabooMax.
*/
void greedy(char help[3],
	    int NbLoop, 
	    int NbIter,
	    int TabooMin,
	    int TabooMax,
	    int Ratio);

/** Algorithme genetique.
    @param NbGen: Nombre de generations (integer)
    @param NbMap: Nombre d'elements dans la population (integer)
    @param SelType:# Selection type :(integer)
    #	0 -> roulette wheel
    #	1 -> stochastic remainder without replacement
    @param ProbaCross: Probabilities of crossover and mutation (float)
    @param ProbaMut: Probabilities of crossover and mutation (float)
    @param EvolFitn:Evolutive Fitness (integer) 
    (the fitness of the same element changes with the generation number)
    
*/
void algogen( char help[3],
	      int NbGen,
	      int NbMap,
	      int SelType,
	      float ProbaCross,
	      float ProbaMut,
	      int EvolFitn);

void cg2tsp(char help[3], char *fileName);
void lkh(char help[3], int nbrun, int backtrack);
void ilkh(char help[3], int nbrun, int collectmaps, double threshold, int cost);
void lkhn(char help[3], int nbrun, int backtrack);
void ilkhn(char help[3], int nbrun, int collectmaps, double threshold, int cost);
void lkhd(char help[3], int nbrun, int backtrack);
void ilkhd(char help[3], int nbrun, int collectmaps, double threshold, int cost);
void lkhl(char help[3], int nbrun, int backtrack);
void ilkhl(char help[3], int nbrun, int collectmaps, double threshold, int cost);
void lkhocb(char help[3], int nbrun, int backtrack);
void ilkhocb(char help[3], int nbrun, int collectmaps, double threshold, int cost);
void lkhocbn(char help[3], int nbrun, int backtrack);
void ilkhocbn(char help[3], int nbrun, int collectmaps, double threshold, int cost);
int mapocb(char help[3], int MapID);

/** pour faire un polish sur la meilleure carte du tas

    @param help pour l'option -h.
*/
void polish(char help[3]);

/** pour faire un polish sur la meilleure carte du tas avec liste
    de marqueurs
    @param MrkList vecteur de marqueurs.
    @param help pour l'option -h.
*/
void polishtest(char help[3], int MrkList[NBM]);

/** pour faire des flips sur la meilleure carte du tas

    @param help pour l'option -h.
    @param WinSize taille de lafentre 
    @param Thres le seuil
    @param Iter flag pour itratif ou non

*/
void flips(char help[3], int WinSize, double Thres, int Iter);

/** to set the equivalence mode of the heap

    @param help pour l'option -h.
    @param flag 0 ou 1
*/
void heapequiset(char help[3], int flag);

/** to set the size of the heap

    @param help pour l'option -h.
    @param hs la taille du tas
*/
void heapsize(char help[3], int NbMap = 5);

/** to get the size of the heap

    @param help to get help.
    @return the size of the heap
*/
int heapsizeget(char help[3]);

/** pour un affichage avec analyse du tas

    @param help pour l'option -h.
    @param NbMrk le nombre de marqueur min permettant de considrer un interval
    @param Blank si 1 carac. blanc pour l'interval sinon l'indice
    @param Comp  si 1 affichage des bornes uniquement
*/
void heaprinto(char help[3], int NbMrk, int Blank, int Comp);

/** pour un affichage du tas

    @param help pour l'option -h.
*/
void heaprint(char help[3]);

/** pour un affichage dtaill du tas

    @param help pour l'option -h.
*/
void heaprintd(char help[3]);

/** pour rcuprer les cartes sous la forme d'une liste

    @param help pour l'option -h.
    @param UnitFlag k ou h
    @param NbMap le nombre de cartes  retourner  partir de la meilleure
    @return llmaplout liste de listes de cartes. 
*/
void heapget(char help[3], char UnitFlag[2], int NbMap, char *****llmapout);

/** Affichage simple d'une carte

    @param help pour l'option -h.
    @param nbmap le numro de la carte dans le tas
*/
void maprint(char help[3], int MapID);

/** Affichage dtaill d'une carte

    @param help pour l'option -h.
    @param nbmap le numro de la carte dans le tas
*/
void maprintd(char help[3], int MapID);

/** Affichage dtaill d'une carte " l'envers"

    @param help pour l'option -h.
    @param nbmap le numro de la carte dans le tas
*/
void maprintdr(char help[3], int MapID);

/** pour rcuprer une carte sous la forme d'une liste

    @param help pour l'option -h.
    @param UnitFlag k ou h
    @param nbmap le numro de la carte dans le tas
    @return lmapout listes de cartes. 
*/

void mapget(char help[3], char UnitFlag[2], int MapID, char ****lmapout);

/** pour rcuprer une carte sous la forme d'une liste

    @param help pour l'option -h.
    @param MapID le numro de la carte dans le tas
    @return lmiout liste d'indices de locus
*/
void mapordget(char help[3], int MapID, int **lmiout);

/** Calcul et affiche les groupes d'un jeu. 
    
    @param help pour l'option -h.
    @param DistThres Seuil de distance.
    @param LODThres Seuil de LOD.
    @return le nombre de groupes trouvs.
*/
int group(char help[3], double DistThres, double LODThres);

/** Rcupration d''un groupe par id de marqueurs. 

    @param help pour l'option -h.
    @param GrID id d'un groupe pralablement calcul
    @param lmiout une liste d'id de marqueurs(out).
*/
void groupget(char help[3], int GrID, int **lmiout);

/** Spcifier le nom du fichier de trace.

    @param help pour l'option -h.
    @param FileName nom de fichier texte.
*/
void cgout(char help[3], char *FileName);

/** Fixer les tolerances de convergence

    @param help pour l'option -h.
    @param Threshold1 seuil de convergence fin
    @param Threshold2 seuil de convergence grossier
*/
void cgtolerance(char help[3], double Threshold1,double Threshold2);

/** Fixer le seuil de fiabilit des cartes frameworks

    @param help pour l'option -h.
    @param Threshold cart de vraisemblance relatif  la meilleure carte
*/
void cgrobustness(char help[3], double Threshold);

/** Inhiber le seuil de fiabilit des cartes frameworks

    @param help pour l'option -h.
*/
void cgnotrobust(char help[3]);

/** Obtenir la version
    
    @param help pour l'option -h.
*/
void cgversion(char help[3]);

/** pour obtenir des informations permettant de faire des stats
   
    @param help pour l'option -h.
    @param LODThres Seuil de LOD.
    @return textout une ligne de texte informative(out). 
*/

void cgstat(char help[3], double LODThres, char **textout);

/** Redmarrage  vide de Cartagene
    
    @param help pour l'option -h.
*/
void cgrestart(char help[3]);

/** Entre la clef de license
    
    @param help pour l'option -h.
    @param Key la clef.
*/
void cglicense(char help[3], char *Key);

/** Arrte une commande
    
    @param help pour l'option -h.
*/
void cgstop(char help[3]);

#endif
