//-----------------------------------------------------------------------------
//                            CarthaGene
//
// Martin Bouchez, Patrick Chabrier, Christine Gaspin and Thomas Schiex. 
// Unite de Biometrie et Intelligence Artificielle, Toulouse, France
//
//  Copyright (C) 1997 INRA (Institut National de la Recherche Agronomique).
//
// $Id: cgsh.i,v 1.57 2004/06/30 08:13:25 degivry Exp $
//
//    This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Module       : cgsh
// Projet       : CartaGene
//
// Description  : Dfinition de "l''interface" de SWIG.
// Ce fichier sert  gnrer le fichier d''interface de SWIG pour Tcl.
// Il sert aussi  gnrer la documentation.
//
// Ce fichier est trs DELICAT  utiliser, car il contient du :
// - code SWIG
// - code C + librairie Tcl pour "amliorer" l''interface.
// - code LaTex pour la documentation utilisateur.
//-----------------------------------------------------------------------------


%title "The CarthaGene Reference Manual"

%module CartaGene

%include typemaps.i

%{
#include "cgsh.h"
#include <signal.h>
Tcl_Interp *linterp;
Tcl_Channel fout;
Tcl_Channel ferr;
extern void hand(int); 
%}

%init %{

linterp = interp;
fout = Tcl_GetStdChannel(TCL_STDOUT);
ferr = Tcl_GetStdChannel(TCL_STDERR);
Tcl_PkgProvide(interp, "CarthaGene", "0.99");
signal(SIGINT, hand);
%}

%{
#ifdef ihm
#define HELPMESS(message, usage, use, description) \
  if (objc > 1) \
  {\
    int temphelp;\
    if ((_arg0 = Tcl_GetStringFromObj(objv[1], &temphelp)) == NULL) \
      return TCL_ERROR; \
    if (strcmp(_arg0,"-u") == 0) \
    {\
      tcl_result = Tcl_GetObjResult(interp);\
      Tcl_SetStringObj(tcl_result,\
	  	     use, -1);\
      return TCL_OK;\
    }\
    if (strcmp(_arg0,"-h") == 0) \
    {\
      tcl_result = Tcl_GetObjResult(interp);\
      Tcl_SetStringObj(tcl_result,\
		     message, -1);\
      return TCL_OK;\
    }\
    if (strcmp(_arg0,"-H") == 0) \
    {\
       sprintf(bouf,"\nUsage : %s\n\nDescription : %s\n\n", usage, description);\
       sprintf(boufi, "puts -nonewline {%s}", bouf); Tcl_Eval(linterp, boufi);\
       if (Fout != NULL) \
         fprintf(Fout,"\nUsage : %s\n\nDescription : %s\n\n", usage, description);\
       fflush(stdout);\
       return TCL_OK;\
    }\
  }
#else
#define HELPMESS(message, usage, use, description) \
  if (objc > 1) \
  {\
    int temphelp;\
    if ((_arg0 = Tcl_GetStringFromObj(objv[1], &temphelp)) == NULL) \
      return TCL_ERROR; \
    if (strcmp(_arg0,"-u") == 0) \
    {\
      tcl_result = Tcl_GetObjResult(interp);\
      Tcl_SetStringObj(tcl_result,\
	  	     use, -1);\
      return TCL_OK;\
    }\
    if (strcmp(_arg0,"-h") == 0) \
    {\
      tcl_result = Tcl_GetObjResult(interp);\
      Tcl_SetStringObj(tcl_result,\
		     message, -1);\
      return TCL_OK;\
    }\
    if (strcmp(_arg0,"-H") == 0) \
    {\
       sprintf(bouf,"\nUsage : %s\n\nDescription : %s\n\n", usage, description);\
       Tcl_Write(fout,bouf,strlen(bouf));\
       fflush(stdout);\
       return TCL_OK;\
    }\
  }
#endif
%}

%typemap(tcl8,check) char help[3]()
{
  if (Fout != NULL)
    {
      int temphelp;
      fprintf(Fout, "\nCG_log>");
      for (int i = 0; i < objc; i++)
	fprintf(Fout, " %s", Tcl_GetStringFromObj(objv[i], &temphelp));
      fprintf(Fout, "\n");
    }
}


//----------------------- la commande dsload

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Load a Biological Data Set.", "dsload [-h | -H | -u | FileName]" ,"dsload FileName" ,"dsload loads a data set. The file format expected for a biologiacl data set is MapMaker like. To load a set of constraint, a specific format is required. The command returns a list containing either the ID of the Biological Data Set, The type of population, the number of individuals and the number of loci,  or the ID of Data Set and the number of constraint loaded.");
}

%typemap(tcl8,ignore) char **textout(char *temp)
{
  $target = &temp;
}

%typemap(tcl8,argout) char **textout
{
  Tcl_Obj *o;
  o = Tcl_NewStringObj(*$source, strlen(*$source));
  delete [] *$source;      
  Tcl_ListObjAppendElement(interp,$target,o);
  if (Fout != NULL)
    {
      int temphelp;
      fprintf(Fout, " %s\n", Tcl_GetStringFromObj($target, &temphelp));
    }
}

void dsload(char help[3], char FileName[256], char **textout);


//----------------------- la commande dsmergen

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Merge two Biological Data Sets.", "dsmergen [-h | -H | -u | SetID1 SetID2]", "dsmergen SetID1 SetID2", "dsmergen merges two Biological Data Set loaded, in a new data set which it creates. The Data Set Tree is updated, the root of the tree points to the new data set. The command returns a list containing either the ID of the Data Set, The type of population, the number of individuals and the number of loci, or the ID of Data Set and the number of constraimt loaded.");
}



void dsmergen(char help[3],int SetID1, int SetID2, char **textout);




//----------------------- la commande dsmergor

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Merge by order two Biological Data Sets.", "dsmergor [-h | -H | -u | SetID1 SetID2]", "dsmergor SetID1 SetID2", "dsmergor merges two Data Set loaded, in a new data set which it creates. This newly created Data Set enables to evaluate an order of makers. But in this case, the position is considered relative two the original Data Set. The Data Set Tree is updated, the root of the tree points to the new data set. The command returns a list containing the ID of the Biological Data Set, The type of population, the number of individuals and the number of loci.");
}



void dsmergor(char help[3],int SetID1, int SetID2, char **textout);




//----------------------- la commande dsinfo

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Summarize the current data sets.", "dsinfo [-h | -H | -u]" ,"dsinfo", "dsinfo prints a table, summarizing the biological informations loaded, Data Set by Data Set");
}



void dsinfo(char help[3]);




//----------------------- la commande dsget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get a list of the current data sets description.", "dsget [-h | -H | -u]" ,"dsget", "dsget returns a list of strings. Each string summarize a Data Set of the current session.");
}

%typemap(tcl8,ignore) char ***ltextout(char **temp)
{
  $target = &temp;
}

%typemap(tcl8,argout) char ***ltextout
{
  int idt = 0;
  while ((*$source)[idt] != 0)
    {
      Tcl_ListObjAppendElement(interp,
			       $target,
			       Tcl_NewStringObj((*$source)[idt],strlen((*$source)[idt])));
      delete (*$source)[idt++];
    }
  delete *$source;
  if (Fout != NULL)
    {
      int temphelp;
    fprintf(Fout, " %s\n", Tcl_GetStringFromObj($target, &temphelp));
    }
}



void dsget(char help[3], char ***ltextout);




//----------------------- la commande mrkinfo

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Tell which data set each marker belongs to.", "mrkinfo [-h | -H | -u]", "mrkinfo", "mrkinfo prints a table, summarizing the biological informations loaded, Marker by Data Marker");
}



void mrkinfo(char help[3]);




//----------------------- la commande mrkselset

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Set the marker list(by Id).", "mrkselset [-h | -H | -u | MrkList]", "mrkselset MrkList", "mrkselset enables to work on a sublist of the loci loaded. By default, each locus is selected.");
}

%typemap(tcl8,in) int [ANY] (int temp[$dim0]) 
{
  int objcount ,i, objet;
  Tcl_Obj **objvect;

  if (Tcl_ListObjGetElements(interp,
                             $source,
                             &objcount,
                             &objvect) == TCL_ERROR)
    return TCL_ERROR;
  
  if (objcount >= $dim0)  
    { 
      interp->result = "Array size mismatch!"; 
      return TCL_ERROR; 
    } 
  
  for (i = 0; i < objcount; i++) 
   { 
     if (Tcl_GetIntFromObj(interp,  
                           objvect[i],   
                           &objet) == TCL_ERROR)  
        return TCL_ERROR; 
     temp[i] = objet; 
   }

  for (i = objcount; i < $dim0; i++)
    temp[i] = 0;
  
  $target = temp;
}



void mrkselset(char help[3], int MrkList[NBM]);




//----------------------- la commande mrkselget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Return the loci selection into a list.", "mrkselget [-h | -H | -u]", "mrkselget", "mrkselget returns a list containing each selected locus. The markers are referenced by Id.");
}

%typemap(tcl8,ignore) int **lmiout(int *temp)
{
  $target = &temp;
}

%typemap(tcl8,argout) int **lmiout
{
  int idm = 0;
  while ((*$source)[idm] != 0)
    Tcl_ListObjAppendElement(interp,
                             $target,
                             Tcl_NewIntObj((*$source)[idm++]));
                             
  delete *$source;
  if (Fout != NULL)
    {
      int temphelp;
      fprintf(Fout, " %s\n", Tcl_GetStringFromObj($target, &temphelp));
    }
}



void mrkselget(char help[3], int **lmiout);




//----------------------- la commande mrkallget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Return all the loci into a list.", "mrkallget [-h | -H | -u]", "mrkallget", "mrkallget returns a list containing each locus. The markers are referenced by Id.");
}



void mrkallget(char help[3], int **lmiout);




//----------------------- la commande mrklod2p

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Print the two points LOD matrix.", "mrklod2p [-h | -H | -u]", "mrklod2p", "mrklod2p prints the two points LOD matrix of the loci selection list.");
}



void mrklod2p(char help[3]);




//----------------------- la commande mrkdouble

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Identifies pairs of markers with compatible typing.", "mrkdouble [-h | -H | -u]", "mrkdouble", "mrkdouble dumps all pairs of markers in the current selection which have compatible typing.");
}



void mrkdouble(char help[3]);




//----------------------- la commande mrkdist2p

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Print the two points distance matrix.", "mrkdist2p [-h | -H | -u | UnitFlag]", "mrkdist2p UnitFlag", "mrkdist2p prints the two points distance matrix of the loci selection list.  The UnitFlag argument allows to choose either the Kosambi(k) unit or the Haldane(h) unit. For radiated hybrids, this flag is not active.");
}



void mrkdist2p(char help[3], char UnitFlag[2]);




//----------------------- la commande mrkfr2p

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Print the two points FR/Breaks matrix.", "mrkfr2p [-h | -H | -u ]", "mrkfr2p", "mrkfr2p prints the two points FR/Breaks(RH) matrix of the loci selection list.  ");
}



void mrkfr2p(char help[3]);




//----------------------- la commande mrkmerge

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Merges two compatible markers.", "mrkmerge [-h | -H | -u | MrkID1 MrkId2]", "mrkmerge MrkID1 MrkId2", "mrkmerge merges two compatibles markers. The result of the merging is stored in the first marker. The other one is untouched and should not be selected.");
}



void mrkmerge(char help[3], int MrkID1, int MrkId2);

//----------------------- la commande mrkmerget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Retrive a list of merged markers.", "mrkmerget [-h | -H | -u]", "mrkmerget", "mrkmerget returns a list of list of ids of loci. The markers who are in the same list were merged together. The first locus of the sublist represents the others.");
}



void mrkmerget(char help[3],char ***ltextout);



//----------------------- la commande mrkname

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get the name of a locus.", "mrkname [-h | -H | -u | MrkID]", "mrkname MrkID", "mrkname returns a string containing the name of a knowned locus. The locus is referenced by its integer Id.");
}



void mrkname(char help[3], int MrkID, char **textout);




//----------------------- la commande mrkid

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get the integer id of a locus.", "mrkid [-h | -H | -u | MrkName]", "mrkid MrkName", "mrkid returns a integer. The locus is referenced by its name.");
}



int mrkid(char help[3], char * MrkName);




//----------------------- la commande mrklast

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get the integer id of the last known locus.", "mrklast [-h | -H | -u]", "mrklast", "mrklast returns the integer id of the last known locus.");
}



int mrklast(char help[3]);


//----------------------- la commande verbset

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Make the search process verbose.", "verbset [-h | -H | -u | Flag]", "verbset Flag", "verbset sets the level of verbosity of the search process. Three levels are available(0, 1 or 2). The verbosity has no effect when the the Quiet Flag is set to 1.");
}



void verbset(char help[3], int Flag);




//----------------------- la commande quietset

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Make the search process quiet.", "quietset [-h | -H | -u | Flag]", "quietset Flag", "quietset makes the search process quiet or not. If the flag is set to 1, nothing is printed during the search process. If the flag is set to 0, the amount of printed information depends of the Verbosity.");
}



void quietset(char help[3], int Flag);




//----------------------- la commande sem

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Compute a Single map (Single EM).", "sem [-h | -H | -u]", "sem", "sem use the order of the selection of loci to compute a single map. This command also try to improve the heap with the resulting map.");
}



void sem(char help[3]);




//----------------------- la commande nicemapd

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide quickly a (nice) map, using the 2-points distances.", "nicemapd [-h | -H | -u]", "nicemapd", "nicemapd uses the two points distance criterion to provide a map. Starting from each locus, the command builds gradually the best neighborhoods maps. At the end of the process, the command tries to insert the best map into the heap.")
}



void nicemapd(char help[3]);




//----------------------- la commande nicemapl

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide quickly a (nice) map, using the 2-points LOD criteria.", "nicemapl [-h | -H | -u]", "nicemapl", "nicemapl uses the two points LOD criterion to provide a map. Starting from each locus, the command builds gradually the best neighborhoods maps. At the end of the process, the command tries to insert the best map into the heap.")
}



void nicemapl(char help[3]);


//----------------------- la commande mfmapd

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide quickly a (nice) map, using the 2-points distances.", "mfmapd [-h | -H | -u]", "mfmapd", "mfmapd uses the two points distance criterion to provide a map. The command builds gradually a map by inserting the best (whose distances are minima) available pairs of markers. At the end of the process, the command tries to insert the map into the heap.")
}



void mfmapd(char help[3]);


//----------------------- la commande mfmapl

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide quickly a (nice) map, using the 2-points LOD criteria.", "mfmapl [-h | -H | -u]", "mfmapl", "mfmapl uses the two points LOD criterion to provide a map. The command builds gradually a map by inserting the best (whose LOD criteria are minima) available pairs of markers. At the end of the process, the command tries to insert the map into the heap.")
}



void mfmapl(char help[3]);


//----------------------- la commande build

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Build maps with two-points informations.", "build [-h | -H | -u | NbMap]", "build NbMap", "build uses the two points LOD score criterion to provide maps. The command starts from a couple of loci. At each step of the algorithme, a locus is choosen and added to the previous map. All the maps correspondind to all the possible positions of the new marker are computed. The NbMap argument enable the command to build several maps at the same time. At the end of the process, the maps improving the current solution are inserted into the heap.")
}



void build(char help[3], int NbMap);




//----------------------- la commande buildfw

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Build a framework map.", "buildfw [-h | -H | -u | keepThres AddThres MrkList MrkTest]", "buildfw  keepThres  AddThres MrkList MrkTest", "buildfw starts either by choosing three loci, or by using the list of loci MrkList. The delta of LOD score of the two best maps you can build with three given loci is used as criterion. The greater the delta is the better the solution is. Then, step by step you try each available locus at each possible position. If there is no delta greater than the AddThres Threshold, the algorithme is stopped. If there is many order which delta of LOD score against the best order is smaller than the KeepThres Threshold, they are keepped for the next step. If the flag MrkTest is set to 0 no test, to 1 each remaining locus is tested on each position of the above construct framework, to 2 each remaining locus in the selection is tested on each position of the framework passed in the third argument (MrkList).")
}



void buildfw(char help[3], double keepThres, double AddThres, int MrkList[NBM], int MrkTest);




//----------------------- la commande annealing

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Find good maps using the annealing algorithm.", "annealing [-h | -H | -u | NbTries InitTemp FinalTemp Cooling]", "annealing NbTries InitTemp FinalTemp Cooling", "annealing uses the multipoint LOD score criterion to provide maps. The command starts form the best map stored into the heap. Each time a better map is found, it is stored into the heap. Two changes and Three changes are randomly applied to the current map. The InitTemp argument correspond to the initial temperature. The FinalTemp argument correspond to the final temperature. The Cooling argument correspond to the cooling speed. The NbTries argument allows the user to increase the number of maps computed at each temperature.");
}



void annealing(char help[3], int NbTries, double InitTemp, double FinalTemp, double Cooling);




//----------------------- la commande greedy

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Find good maps using the greedy algorithm.", "greedy [-h | -H | -u | NbLoop NbIter TabooMin TabooMax Anytime]", "greedy NbLoop NbIter TabooMin TabooMax Anytime", "greedy uses the multipoint LOD score criterion to provide maps. The command starts form the best map stored into the heap. Each time a better map is found, it is stored into the heap. The TabooMin argument correspond to the minimum size of the taboo list. The TabooMax argument correspond to the maximum size of the taboo list.The NbLoop argument correspond to the number of main loop to provide. The NbIter argument allows the user to increase the number of maps computed at each loop. The Anytime ratio controls the tradeoff between search speed and solution quality.");
}



void greedy(char help[3],int NbLoop, int NbIter, int TabooMin, int TabooMax, int Anytime);




//----------------------- la commande algogen

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Find good maps using the genetic algorithm.", "algogen [-h | -H | -u | NbGen NbMap SelType ProbaCross ProbaMut EvolFitn]", "algogen NbGen NbMap SelType ProbaCross ProbaMut EvolFitn", "algogen uses the multipoint LOD criterion to provide maps. The command starts either form the best maps stored into the heap or from generated maps. Each time a better map is found, it is stored into the heap. The Nbgen argument correspond to the number of generation. The NbMap argument correspond to the size of the population. In case NbMap is set to 0, the command use the heap to initialize the first generation. The SelType argument correspond to the type of selection( 0 for the roulette wheel, 1 for the stochastic remainder without replacement). The ProbaCross argument correspond to the probability of crossing-over. The ProbaMut argument correspond to the probability of mutation. The EvolFitn is a flag to set the evolutive fitness(0 or 1).");
}



void algogen(char help[3], 
             int NbGen,
             int NbMap,
             int SelType,
             float ProbaCross,
             float ProbaMut,
             int EvolFitn);


//----------------------- la commande cg2tsp

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Convert current marker selection to TSP.", "cg2tsp [-h | -H | -u | FileName]" ,"cg2tsp FileName" ,"cg2tsp converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem whose file name is given as parameter.");
}

void cg2tsp(char help[3], char FileName[256]);


//----------------------- la commande lkh

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using 2-points loglikelihoods and Lin-Kernighan heuristic.", "lkh [-h | -H | -u | NbRun CollectMaps]" ,"lkh NbRun CollectMaps" ,"lkh converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic. If CollectMaps >= 0 then every tour found by LKH is inserted into the CarthaGene heap. Moreover, if positive, CollectMaps controls the backtrack move type of LKH (possible values are 0, 2, 3, 4 and 5).");
}

void lkh(char help[3], int nbrun, int backtrack);

//----------------------- la commande ilkh

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using dynamically updated 2-points loglikelihoods and Lin-Kernighan heuristic.", "ilkh [-h | -H | -u | NbRun CollectMaps Threshold Penalty]" ,"ilkh NbRun CollectMaps" ,"ilkh converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic, modifying the TSP objective function accordingly to the multipoint probabilistic parameters. If CollectMaps = 1 then every tour found by LKH is inserted into the CarthaGene heap, otherwise set CollectMaps to zero. Threshold specifies that Penalty should be added to a TSP edge cost if the multipoint Theta minus the 2-points Theta is greater than Threshold. Try e.g. ilkh 20 0 0.01 100.");
}

void ilkh(char help[3], int nbrun, int collectmaps, double threshold, int cost);


//----------------------- la commande lkhn

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using normalized 2-points loglikelihoods and Lin-Kernighan heuristic.", "lkhn [-h | -H | -u | NbRun CollectMaps]" ,"lkhn NbRun CollectMaps" ,"lkhn converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic. If CollectMaps >= 0 then every tour found by LKH is inserted into the CarthaGene heap. Moreover, if positive, CollectMaps controls the backtrack move type of LKH (possible values are 0, 2, 3, 4 and 5).");
}

void lkhn(char help[3], int nbrun, int backtrack);

//----------------------- la commande ilkhn

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using dynamically updated normalized 2-points loglikelihoods and Lin-Kernighan heuristic.", "ilkhn [-h | -H | -u | NbRun CollectMaps Threshold Penalty]" ,"ilkhn NbRun CollectMaps" ,"ilkhn converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic, modifying the TSP objective function accordingly to the multipoint probabilistic parameters. If CollectMaps = 1 then every tour found by LKH is inserted into the CarthaGene heap, otherwise set CollectMaps to zero. Threshold specifies that Penalty should be added to a TSP edge cost if the multipoint Theta minus the 2-points Theta is greater than Threshold. Try e.g. ilkhn 20 0 0.01 100.");
}

void ilkhn(char help[3], int nbrun, int collectmaps, double threshold, int cost);


//----------------------- la commande lkhd

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using 2-points distances and Lin-Kernighan heuristic.", "lkhd [-h | -H | -u | NbRun CollectMaps]" ,"lkhd NbRun CollectMaps" ,"lkhd converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic. If CollectMaps >= 0 then every tour found by LKH is inserted into the CarthaGene heap. Moreover, if positive, CollectMaps controls the backtrack move type of LKH (possible values are 0, 2, 3, 4 and 5).");
}

void lkhd(char help[3], int nbrun, int backtrack);

//----------------------- la commande ilkhd

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using dynamically updated 2-points distances and Lin-Kernighan heuristic.", "ilkhd [-h | -H | -u | NbRun CollectMaps Threshold Penalty]" ,"ilkhd NbRun CollectMaps" ,"ilkhd converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic, modifying the TSP objective function accordingly to the multipoint probabilistic parameters. If CollectMaps = 1 then every tour found by LKH is inserted into the CarthaGene heap, otherwise set CollectMaps to zero. Threshold specifies that Penalty should be added to a TSP edge cost if the multipoint Theta minus the 2-points Theta is greater than Threshold. Try e.g. ilkhd 20 0 0.01 100.");
}

void ilkhd(char help[3], int nbrun, int collectmaps, double threshold, int cost);


//----------------------- la commande lkhl

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using 2-points LOD criteria and Lin-Kernighan heuristic.", "lkhl [-h | -H | -u | NbRun CollectMaps]" ,"lkhl NbRun CollectMaps" ,"lkhl converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic. If CollectMaps >= 0 then every tour found by LKH is inserted into the CarthaGene heap. Moreover, if positive, CollectMaps controls the backtrack move type of LKH (possible values are 0, 2, 3, 4 and 5).");
}

void lkhl(char help[3], int nbrun, int backtrack);

//----------------------- la commande ilkhl

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using dynamically updated 2-points LOD criteria and Lin-Kernighan heuristic.", "ilkhl [-h | -H | -u | NbRun CollectMaps Threshold Penalty]" ,"ilkhl NbRun CollectMaps" ,"ilkhl converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic, modifying the TSP objective function accordingly to the multipoint probabilistic parameters. If CollectMaps = 1 then every tour found by LKH is inserted into the CarthaGene heap, otherwise set CollectMaps to zero. Threshold specifies that Penalty should be added to a TSP edge cost if the multipoint Theta minus the 2-points Theta is greater than Threshold. Try e.g. ilkhl 20 0 0.01 100.");
}

void ilkhl(char help[3], int nbrun, int collectmaps, double threshold, int cost);


//----------------------- la commande lkhocb

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using 2-points Obligate Chromosome Breaks criteria and Lin-Kernighan heuristic.", "lkhocb [-h | -H | -u | NbRun CollectMaps]" ,"lkhocb NbRun CollectMaps" ,"lkhocb converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic. If CollectMaps >= 0 then every tour found by LKH is inserted into the CarthaGene heap. Moreover, if positive, CollectMaps controls the backtrack move type of LKH (possible values are 0, 2, 3, 4 and 5).");
}

void lkhocb(char help[3], int nbrun, int backtrack);

//----------------------- la commande ilkhocb

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using dynamically updated 2-points Obligate Chromosome Breaks criteria and Lin-Kernighan heuristic.", "ilkhocb [-h | -H | -u | NbRun CollectMaps Threshold Penalty]" ,"ilkhocb NbRun CollectMaps" ,"ilkhocb converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic, modifying the TSP objective function accordingly to the multipoint probabilistic parameters. If CollectMaps = 1 then every tour found by LKH is inserted into the CarthaGene heap, otherwise set CollectMaps to zero. Threshold specifies that Penalty should be added to a TSP edge cost if the multipoint Theta minus the 2-points Theta is greater than Threshold. Try e.g. ilkhocb 20 0 0.01 100.");
}

void ilkhocb(char help[3], int nbrun, int collectmaps, double threshold, int cost);


//----------------------- la commande lkhocbn

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using normalized 2-points Obligate Chromosome Breaks criteria and Lin-Kernighan heuristic.", "lkhocbn [-h | -H | -u | NbRun CollectMaps]" ,"lkhocbn NbRun CollectMaps" ,"lkhocbn converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic. If CollectMaps >= 0 then every tour found by LKH is inserted into the CarthaGene heap. Moreover, if positive, CollectMaps controls the backtrack move type of LKH (possible values are 0, 2, 3, 4 and 5).");
}

void lkhocbn(char help[3], int nbrun, int backtrack);

//----------------------- la commande ilkhocbn

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Provide a (nice) map, using dynamically updated normalized 2-points Obligate Chromosome Breaks criteria and Lin-Kernighan heuristic.", "ilkhocbn [-h | -H | -u | NbRun CollectMaps Threshold Penalty]" ,"ilkhocbn NbRun CollectMaps" ,"ilkhocbn converts the current marker selection and associated genetic / Radiated Hybrid data into a Traveling Salesman Problem which is solved using Keld Helsgaun's LKH software. Repeats NbRun times the heuristic, modifying the TSP objective function accordingly to the multipoint probabilistic parameters. If CollectMaps = 1 then every tour found by LKH is inserted into the CarthaGene heap, otherwise set CollectMaps to zero. Threshold specifies that Penalty should be added to a TSP edge cost if the multipoint Theta minus the 2-points Theta is greater than Threshold. Try e.g. ilkhocbn 20 0 0.01 100.");
}

void ilkhocbn(char help[3], int nbrun, int collectmaps, double threshold, int cost);


//----------------------- la commande mapocb

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Obligate Chromosome Breaks of a map.", "mapocb [-h | -H | -u | MapID]", "mapocb", "mapocb returns the value of the 2-points Obligate Chromosome Breaks criterion. The MapID should be the identifier of a map stored into the heap.");
}



int mapocb(char help[3], int MapID);


//----------------------- la commande polish

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Try to improve the best map by polishing.", "polish [-h | -H | -u]", "polish", "polish starts form the best map of the heap, an try to improve the solution. Taking each locus, one after the other, this commands tests all the possible position of this locus on the map, keeping the order of the other loci. Each time the solution is improved, the command tries to store the map into the heap.");
}



void polish(char help[3]);




//----------------------- la commande polishtest

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Try to improve the map giving with markers by polishing.", "polish [-h | -H | -u | MrkList]", "polish", "polishtest starts from the map giving on the command, an try to improve the solution. Taking each locus, one after the other, this commands tests all the possible position of this locus on the map, keeping the order of the other loci.");
}

%typemap(tcl8,in) int [ANY] (int temp[$dim0]) 
{
  int objcount ,i, objet;
  Tcl_Obj **objvect;

  if (Tcl_ListObjGetElements(interp,
                             $source,
                             &objcount,
                             &objvect) == TCL_ERROR)
    return TCL_ERROR;
  
  if (objcount >= $dim0)  
    { 
      interp->result = "Array size mismatch!"; 
      return TCL_ERROR; 
    } 
  
  for (i = 0; i < objcount; i++) 
   { 
     if (Tcl_GetIntFromObj(interp,  
                           objvect[i],   
                           &objet) == TCL_ERROR)  
        return TCL_ERROR; 
     temp[i] = objet; 
   }

  for (i = objcount; i < $dim0; i++)
    temp[i] = 0;
  
  $target = temp;
}



void polishtest(char help[3], int MrkList[NBM]);




//----------------------- la commande flips

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Try to improve the best map by flipping.", "flips [-h | -H | -u | WinSize Thres Iter]", "flips WinSize Thres Iter", "flips starts from the best map of the heap and try to improve the solution. Inside a window pushed on all the positions of the map, all the permutation are tested. Each time a better solution is found, the process prints it, and tries to insert it into the heap. The Winsize argument allows you to specify the window size. The Thres argument is a LOD threshold, that enables to see worse maps, that differs less than this value from the best map. If the iter flag is set to 1, each time a better solution is found, the command will start again from the best map.");
}



void flips(char help[3], int WinSize, double Thres, int Iter);


//----------------------- la commande heapsize

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Resize the heap.", "heapsize [-h | -H | -u | NbMap]", "heapsize NbMap", "heapsize enable to resize the map container. If the heap of maps is filled, the best maps are preserved.");
}



void heapsize(char help[3], int NbMap);

//----------------------- la commande heapequiset

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("enable or disable the equivalence mode of the heap.", "heapequiset [-h | -H | -u| Flag]", "heapequiset Flag", "heapequiset enable to switch the equivalence mode of the heap. If the flag is set to 1 the equivalence mode is activated. Dealing with data sets merged by order has the side effect that many order encountered during a build or search process are equivalent. This mode propose to store only one representant for each class.");
}


void heapequiset(char help[3], int flag);



//----------------------- la commande heapsizeget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("get the size of the heap.", "heapsizeget [-h | -H | -u]", "heapsizeget", "heapsizeget enable to get the size of the the map container.");
}


int heapsizeget(char help[3]);



//----------------------- la commande heaprinto

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Display the heap of maps by privileging the order of the markers.", "heaprinto [-h | -H | -u |NbMrk Blank Comp]", "heaprinto NbMrk Blank Comp", "heaprinto displays for each map the positions of markers on the best map. The NbMrk allows you to materialize consecutive differences, if set to 0 all the positions are printed. If the Blank argument is set to 1, the positions inside a interval are replaced by -. Set to 1, the Comp argument makes the lines shorter by removing blanks.");
}



void heaprinto(char help[3], int NbMrk, int Blank, int Comp);




//----------------------- la commande heaprint

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Display the heap sorted.", "heaprint [-h | -H | -u]", "heaprint", "heaprint prints each map stored into the heap by order of LOD score. The information provided is limited to the order and the score.");
}



void heaprint(char help[3]);




//----------------------- la commande heaprintd

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Display the heap in detail, sorted.", "heaprintd [-h | -H | -u]", "heaprintd", "heaprintd  prints in detail each map stored into the heap by order of LOD score. All the information available is provided.");
}



void heaprintd(char help[3]);




//----------------------- la commande heapget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get the heap, in a list.", "heapget [-h | -H | -u | UnitFlag NbMap]", "heapget UnitFlag NbMap", "heapget returns information about the NbMap best maps into a list.");
}

%typemap(tcl8,ignore) char *****llmapout(char ****temp)
{
  $target = &temp;
}

%typemap(tcl8,argout) char *****llmapout
{
  int idlmap = 0;
  int idmap = 0;
  int idval = 0;
  Tcl_Obj * maplist;
  Tcl_Obj * lmaplist;
  
  if (*$source != NULL)
    {      
      idlmap = 0;
      while ((*$source)[idlmap] != NULL)
	{
	  lmaplist = Tcl_NewListObj(0,NULL);
	  idmap = 0;
	  Tcl_ListObjAppendElement(interp,
				   lmaplist,
				   Tcl_NewStringObj((*$source)[idlmap][idmap][0],
						    strlen((*$source)[idlmap][idmap][0])));
	  Tcl_ListObjAppendElement(interp,
				   lmaplist,
				   Tcl_NewStringObj((*$source)[idlmap][idmap][1],
						    strlen((*$source)[idlmap][idmap][1])));
	  idmap++;
	  while ((*$source)[idlmap][idmap] != NULL)
	    {    
	      maplist = Tcl_NewListObj(0,NULL);
	      idval = 0;
	      while ((*$source)[idlmap][idmap][idval] != NULL)
		{
		  Tcl_ListObjAppendElement(interp,
					   maplist,
					   Tcl_NewStringObj((*$source)[idlmap][idmap][idval],
							    strlen((*$source)[idlmap][idmap][idval])));
		  delete [] (*$source)[idlmap][idmap][idval++];
		}
	      Tcl_ListObjAppendElement(interp,
				       lmaplist,
				       maplist);
	      delete [] (*$source)[idlmap][idmap++];
	    }
	  Tcl_ListObjAppendElement(interp,
				   $target,
				   lmaplist);
	  delete [] (*$source)[idlmap++];
	}
      delete [] (*$source);
    }
  else
    Tcl_ListObjAppendElement(interp,
			     $target,
			     Tcl_NewStringObj("",
					      1));
  if (Fout != NULL)
    {
      int temphelp;
      fprintf(Fout, " %s\n", Tcl_GetStringFromObj($target, &temphelp));
    }
}
  


void heapget(char help[3], char UnitFlag[2], int NbMap, char *****llmapout);


//----------------------- la commande maprint

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Print a map.", "maprint [-h | -H | -u | MapID]", "maprint", "maprint prints each locus at his position on the maps, and the log10-likelihood. The MapID should be the identifier of a map stored into the heap.");
}



void maprint(char help[3], int MapID);




//----------------------- la commande maprintd

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Print a map, in detail.", "maprintd [-h | -H | -u | MapID]", "maprintd", "maprintd prints all the information about the map. The MapID should be the identifier of a map stored into the heap.");
}



void maprintd(char help[3], int MapID);




//----------------------- la commande maprintdr

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Print a map reverse, in detail.", "maprintdr [-h | -H | -u | MapID]", "maprintdr", "maprintdr prints all the information about the map. The MapID should be the identifier of a map stored into the heap.");
}



void maprintdr(char help[3], int MapID);




//----------------------- la commande mapget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get a map, in a list.", "mapget [-h | -H | -u | UnitFlag MapID]", "mapget UnitFlag MapID", "mapget returns information about the map. The MapID should be the identifier of a map stored into the heap. The returned list contains the MapID, a global LOD score, as well as as many sublists than of partial orders. Each sublist contains the SetId, the partial LOD Score as well as the name of each loci and their respective relative position. The UnitFlag argument allows to choose either the Kosambi(k) unit or the Haldane(h) unit. For radiated hybrids, this flag is not active.");
}

%typemap(tcl8,ignore) char ****lmapout(char ***temp)
{
  $target = &temp;
}

%typemap(tcl8,argout) char ****lmapout
{
  int idmap = 0;
  int idval = 0;
  Tcl_Obj * maplist;
  
  if (*$source != NULL)
    {      
      Tcl_ListObjAppendElement(interp,
			       $target,
			       Tcl_NewStringObj((*$source)[idmap][idval],
						strlen((*$source)[idmap][idval])));
      Tcl_ListObjAppendElement(interp,
			       $target,
			       Tcl_NewStringObj((*$source)[idmap][1],
						strlen((*$source)[idmap][1])));
      idmap++;
      while ((*$source)[idmap] != NULL)
	{
	  maplist = Tcl_NewListObj(0,NULL);
	  idval = 0;
	  while ((*$source)[idmap][idval] != NULL)
	    {
	      Tcl_ListObjAppendElement(interp,
				       maplist,
				       Tcl_NewStringObj((*$source)[idmap][idval],
							strlen((*$source)[idmap][idval])));
	      delete [] (*$source)[idmap][idval++];
	    }
	  Tcl_ListObjAppendElement(interp,
				   $target,
				   maplist);
	  delete [] (*$source)[idmap++];
	}                           
      delete [] *$source;      
    } 
  else
    Tcl_ListObjAppendElement(interp,
			     $target,
			     Tcl_NewStringObj("",
					      1));
  if (Fout != NULL)
    {
      int temphelp;
      fprintf(Fout, " %s\n", Tcl_GetStringFromObj($target, &temphelp));
    }
}
  


void mapget(char help[3], char UnitFlag[2], int MapID, char ****lmapout);



//----------------------- la commande mapordget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get a order of loci from a map, in a list.", "mapordget [-h | -H | -u | MapID]", "mapordget MapID", "mapordget returns the loci order of the map. The MapID should be the identifier of a map stored into the heap. The returned list contains loci by Id.");
}



void mapordget(char help[3], int MapID, int **lmiout);



//----------------------- la commande group

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Identify linkage groups.", "group [-h | -H | -u | DistThres LODThres]", "group DistThres LODThres", "group tries to identify the linkage groups of a Data Set by a two-points analysis. The command is applied to the selection of loci. The DistThres argument enables to set the distance threshold(a maximum). The LODThres argument enables to set LOD threshold(a minimum). The command prints each groups, and returns the number of identified groups. The groups are stored.")
}



int group(char help[3], double DistThres, double LODThres);




//----------------------- la commande groupget

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get a group by markers ID.", "groupget [-h | -H | -u | GrID]", "groupget GrID", "groupget gets a group previously detected by the command group. The GrID argument corrrespond to the group identifier. The group is returned as a list of loci index.")
}

void groupget(char help[3], int GrID, int **lmiout);


//----------------------- la commande cgout

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Save the output to a file.", "dsload [-h | -H | -u | FileName]" ,"dsload FileName" ,"cgout enables to save the session output in a text file. If the file already exist, the ouput is appened. To stop the process, you need to call cgout one more time with a emtpy string.");
}

void cgout(char help[3], char FileName[256]);


//----------------------- la commande tolerance

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Sets EM convergence thresholds.", "cgtolerance [-h | -H | -u | Threshold1 Threshold2]", "cgtolerance Threshold1 Threshold2", "cgtolerance sets the thresholds used inside EM, the recombination/breakage/retention estimation algorithm. The first threshold is used for fine convergence for final maps, the second one is used for raw convergence in ordering commands to decide if further iterations are worth the effort.")
}



void cgtolerance(char help[3], double Threshold1, double Threshold2);


//----------------------- la commande robustesse

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Sets reliability threshold for framework maps.", "cgrobustness [-h | -H | -u | Threshold]", "cgrobustness Threshold", "cgrobustness sets the threshold used to verify if the current best map is robust or not. A map M is not robust if there exists another map M' such that L(M') > L(M) + Threshold, where L(M) is the multipoint log likelihood of M. A common value for Threshold is -3.0. Each time a new map is inserted into the heap, the system verifies if the current best map is still robust. If not then it will stop any running search procedure.")
}



void cgrobustness(char help[3], double Threshold);


//----------------------- la commande inhiber robustesse

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Disable reliability test of framework maps.", "cgnotrobust [-h | -H | -u]", "cgnotrobust", "cgnotrobust disables the reliability test of framework maps. It is equivalent to call cgrobustness with Threshold equal to 1e100.")
}



void cgnotrobust(char help[3]);




//----------------------- la commande cgversion

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get the version.", "cgversion [-h | -H | -u]", "cgversion", "cgversion gives the version of the software(CartaGene part).");
}



void cgversion(char help[3]);




//----------------------- la commande cgstat

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Get some performance information.", "cgstat [-h | -H | -u | LODThres]", "cgstat LODThres", "cgstat returns informations usefull to provide performance statistics. The LODThres argument is used to count the maps close to the best one inside this threshold.");
}



void cgstat(char help[3], double LODThres, char **textout);




//----------------------- la commande cgrestart

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Reset the application.", "cgrestart [-h | -H | -u]", "cgrestart", "cgrestart erases all computed information and loaded  information, of the system.");
}



void cgrestart(char help[3]);




//----------------------- la commande cglicense

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("Deprecated command.", "cglicense[-h | -H | -u | Key]", "cglicense Key", "cglicense is a deprecated command.");
}



void cglicense(char help[3], char* Key);




//----------------------- la commande cgstop

%typemap(tcl8,ignore) char help[3]()
{
  HELPMESS("To stop a running command, type ctrl-c.", "cgstop[-h | -H | -u ]", "cgstop", "The ctrl-c sequence will enable the user to emit a request of interruption of the current command. The request will be handled next time a map is computed.");
}

 
void cgstop(char help[3]);



